--*******************************************
-- Desc: 控制类
-- File: ControlLayer.lua
-- Author: zhouhaifeng
-- Date: 2014-03-09 15:44:01
--*******************************************

local PauseLayer = import(".PauseLayer")
local ContinueLayer = import(".ContinueLayer")
local TaskInfoLayer = import(".TaskInfoLayer")
local BuyConfirmLayer = import(".BuyConfirmLayer1")
local ControlLayer = class("ControlLayer",function()
    return display.newLayer("ControlLayer")
end)

local handler = nil

local ListenEvents = {
    "GAME_START",
    "GAME_WIN",
    "GAME_FAILED",
    "PLAYER_DEAD",
    "PLAYER_REBORN",
    "PLAYER_ATTACKED",
    "PLAYER_FLY",
    "MONSTER_DEAD",
    "CAR_DISTORYED",
    "TASK_FINISHED"
}

--[[
初始化记录值
]]
function ControlLayer:setupRecord()
    EquipSystem:setUseTime()
    EffectHelper.reset()

    self:resetRecord()

    self.isForeverMode = LevelSystem._level.isForeverMode

end

function ControlLayer:resetRecord()

    self.record = {}

    self.record.startTime = 0
    self.record.distance = 0
    self.record.numberOfkillEnemy = 0               --杀敌数
    self.record.numberOfkillBoss = 0                --杀死Boss次数
    self.record.numberOfkillAdvBoss = 0             --杀死高级Boss次数
    self.record.numberOfkillSuperBoss = 0           --杀死SuperBoss次数
    self.record.numberOfhitCars = 0                 --摧毁车数
    self.record.numberOfhitEnemy = 0                --撞敌数
    self.record.maxNumberOfCombs = 0                --最大连击数
    self.record.maxNumberOfSpeed = 0                --达到的最大速度
    self.record.numberOfHittings = 0                --被怪物击中次数
    self.record.numberOfFly = 0                     --跳跃次数
    self.record.normalOfDistance = 0
    self.record.speedOfDistance = 0
    self.record.numberOfDeads = 0                   --死亡次数
    self.record.numberOfGlod = 0                    --获得的金币
    self.record.maxTimeOfForeverMode = 0            --无尽模式最长时间
    self.record.reloadTimes = 0

    GameSystem._record.preGameInfo = GameSystem._record.lastGameInfo
    GameSystem._record.lastGameInfo = self.record
end

--[[
初始化
params: 地图信息
]]
function ControlLayer:ctor()

    self:setupRecord()

    self.layer = display.newLayer()
    self:addChild(self.layer)

    self.bulletBG = display.newSprite(
        "ui/gameui/ui_bullet_bg.png",
        display.width - 140,
        display.top - 62
    )
    self.layer:addChild(self.bulletBG)

    self.bulletFlash = display.newSprite(
        "ui/gameui/ui_bullet_flash.png",
        0,
        0
    )
    self.bulletFlash:setVisible(false)
    self.bulletFlash:setAnchorPoint(ccp(0,0))
    self.bulletBG:addChild(self.bulletFlash)


    self.buttletsLabel = ui.newAtlasLabel({
        text = ":00",
        start = '0',
        image = "ui/gameui/number_bar_bullet.png",
        align = ui.TEXT_ALIGN_LEFT,
        w = 12,
        h = 17,
        x = 26,
        y = 6,
    })
    self.bulletBG:addChild(self.buttletsLabel)

    self.bulletTitle = display.newSprite("ui/gameui/ui_bullet_title.png", 20, 15)
    self.bulletBG:addChild(self.bulletTitle)

    self.dashboardBg = display.newSprite(
        "ui/gameui/ui_game_speed.png",
        display.width - 105,
        display.top - 65)
    self.layer:addChild(self.dashboardBg)

    self.dashboardPin = display.newSprite(
        "ui/gameui/ui_game_speed_pin.png",
        150,
        66
    )
    self.dashboardPin:setAnchorPoint(ccp(1,0.5))
    self.dashboardBg:addChild(self.dashboardPin)

    self.monsterLabel = ui.newAtlasLabel({
        text = "00000",
        start = '0',
        image = "ui/gameui/number_bar_monster.png",
        w = 15,
        h = 22,
        x = 100,
        y = 100,
    })
    self.monsterLabel:setAnchorPoint(ccp(1,0.5))
    self.dashboardBg:addChild(self.monsterLabel)

    self.distanceLabel = ui.newAtlasLabel({
        text = string.format("%5d",0),
        start = '0',
        image = "ui/gameui/number_bar_distance.png",
        w = 13,
        h = 15,
        x = 185,
        y = 12,
    })
    self.distanceLabel:setAnchorPoint(ccp(1,0.5))
    self.dashboardBg:addChild(self.distanceLabel)

    self.comboBG = display.newSprite(
        "ui/gameui/ui_game_combo.png",
        display.cx,
        display.top - 80
    )
    self.layer:addChild(self.comboBG)

    self.comboNum = ui.newAtlasLabel({
        text = 0,
        start = '0',
        image = "ui/gameui/ui_game_combo_number.png",
        w = 32,
        h = 40,
        x = 230,
        y = 20,
    })
    self.comboNum:setAnchorPoint(ccp(0,0.5))
    self.comboBG:addChild(self.comboNum)
    self.comboBG:setVisible(false)

    self.gameHPBG = display.newSprite(
        "ui/gameui/ui_hp_bg.png",
        display.cx,
        display.top - 25
    )
    self.layer:addChild(self.gameHPBG)

    self.gameHP = CCProgressTimer:create(CCSprite:create("ui/gameui/ui_hp_progress.png"))
    self.gameHP:setType(kCCProgressTimerTypeBar)
    self.gameHP:setPosition(ccp(85,18))
    self.gameHP:setMidpoint(ccp(0,1))
    self.gameHP:setBarChangeRate(ccp(1,0))
    self.gameHP:setPercentage(100)
    self.gameHPBG:addChild(self.gameHP)

    self.coldProgress = CCProgressTimer:create(CCSprite:create("ui/gameui/ui_reload_mask.png"))
    self.coldProgress:setType(kCCProgressTimerTypeBar)
    self.coldProgress:setPosition(ccp(80,5))
    self.coldProgress:setMidpoint(ccp(0,1))
    self.coldProgress:setBarChangeRate(ccp(1,0))
    self.coldProgress:setPercentage(0)
    self.bulletBG:addChild(self.coldProgress)

    --sneakyButton
    self.speedButton = CCNodeExtend.extend(CCSneakyButton:create())
    self.speedButton:initWithRect(CCRectMake(0, 0, 44, 44))
    self.speedButton:setIsHoldable(true)
    self.skinSpeedButton = CCSneakyButtonSkinnedBase:create()
    self.skinSpeedButton:setPosition(ccp(display.right - 160, display.bottom + 60))
    self.skinSpeedButton:setDefaultSprite(CCSprite:create("ui/gameui/ui_game_up_0.png"))
    self.skinSpeedButton:setPressSprite(CCSprite:create("ui/gameui/ui_game_up_1.png"))
    self.skinSpeedButton:setActivatedSprite(CCSprite:create("ui/gameui/ui_game_up_1.png"))
    self.skinSpeedButton:setButton(self.speedButton);
    self:addChild(self.skinSpeedButton);

    self.shootButton = CCNodeExtend.extend(CCSneakyButton:create())
    self.shootButton:initWithRect(CCRectMake(0, 0, 44, 44))
    self.shootButton:setIsHoldable(true)
    self.skinShootButton = CCSneakyButtonSkinnedBase:create()
    self.skinShootButton:setPosition(ccp(display.right - 60 , display.bottom + 60))
    self.skinShootButton:setDefaultSprite(CCSprite:create("ui/gameui/ui_game_fire_0.png"))
    self.skinShootButton:setPressSprite(CCSprite:create("ui/gameui/ui_game_fire_1.png"))
    self.skinShootButton:setActivatedSprite(CCSprite:create("ui/gameui/ui_game_fire_1.png"))
    self.skinShootButton:setButton(self.shootButton);
    self:addChild(self.skinShootButton);

    self.no2Button = ui.newImageMenuItem({
        image = "ui/gameui/ui_no2_n.png",
        imageSelected = "ui/gameui/ui_no2_h.png",
        imageDisabled = "ui/gameui/ui_no2_h.png",
        x = display.right - 260,
        y = display.bottom + 60,
        listener = function()
            audio.playEffect(R.SOUND_BUTTON)
            local player = Global:get("player")
            if player.config.isDead then
                return false
            end

            local num = EquipSystem:getPropNum(4003)
            if num > 0 then

                EquipSystem:setPropNum(4003, num - 1)
                self.no2Label:setString(string.format(":%02d",EquipSystem:getPropNum(4003)))

                player:playNo2SpeedUp()

                player.config.currentSpeed = player.config.currentSpeed * 2
                player.config.hasCrashBarrier = true
                player.config.isNo2SpeedUp = true
                self.no2Button:setEnabled(false)
                self.no2Mask:runAction(CCSequence:createWithTwoActions(CCProgressFromTo:create(5 ,100, 0),CCCallFunc:create(function()
                    local player = Global:get("player")
                    player.speedupEffect:removeFromParentAndCleanup(true)
                    player.fireEffect:removeFromParentAndCleanup(true)
                    player.speedupEffect = nil
                    player.fireEffect = nil
                    player:showMotoEffect()
                    player.config.currentSpeed = player.config.currentSpeed/2
                    if player.car.itemId == 1006 then
                        player.config.hasCrashBarrier = true
                    else
                        player.config.hasCrashBarrier = false
                    end
                    player.config.isNo2SpeedUp = false
                    self.no2Button:setEnabled(true)
                end)))

            else
                CCLuaLog("购买了3个")
                self:performWithDelay(function()
                    display.pause()
                end, 0.5)
                self.pushLayer = BuyConfirmLayer.new()
                self.pushLayer:setItemIdToBuy(4003,"ui/buy/huoli_5.png",function()
                    end)
                self:addChild(self.pushLayer)

            end
        end,
    })

    self.no2Mask = CCProgressTimer:create(CCSprite:create("ui/gameui/fire_cold_mask.png"))
    self.no2Mask:setPosition(ccp(36,38))
    self.no2Mask:setAnchorPoint(ccp(0.5,0.5))
    self.no2Mask:setPercentage(0)
    self.no2Mask:setType(kCCProgressTimerTypeRadial)
    self.no2Button:addChild(self.no2Mask)

    self.no2Label = ui.newAtlasLabel({
        text = string.format(":%02d",EquipSystem:getPropNum(4003)),
        start = '0',
        image = "ui/gameui/number_bar_bullet.png",
        align = ui.TEXT_ALIGN_LEFT,
        w = 12,
        h = 17,
        x = 18,
        y = -15,
    })
    self.no2Button:addChild(self.no2Label)

    self.bulletsButton = ui.newImageMenuItem({
        image = "ui/gameui/ui_bullets_n.png",
        imageSelected = "ui/gameui/ui_bullets_h.png",
        imageDisabled = "ui/gameui/ui_bullets_h.png",
        x = display.right - 340,
        y = display.bottom + 60,
        listener = function()
            audio.playEffect(R.SOUND_BUTTON)
            local player = Global:get("player")
            if player.config.isDead then
                return false
            end

            local num = EquipSystem:getPropNum(4001)
            if num > 0 then
                EquipSystem:setPropNum(4001, num - 1)
                self.bulletsLabel:setString(string.format(":%02d",EquipSystem:getPropNum(4001)))

                self.bulletFlash:setVisible(true)
                self.bulletFlash:runAction(CCBlink:create(15, 30))

                self.bulletsMask:runAction(CCSequence:createWithTwoActions(CCProgressFromTo:create(15 ,100, 0),CCCallFunc:create(function()
                    local player = Global:get("player")
                    player.config.isBulletInfinite = false
                    self.bulletsButton:setEnabled(true)
                    self.bulletFlash:setVisible(false)
                    if GameSystem:getGuideId() == kGUIDE_BULLETS then
                        self:checkGuide()
                    end
                end)))
            else
                CCLuaLog("购买了3个")
                self:performWithDelay(function()
                    display.pause()
                end, 0.5)
                self.pushLayer = BuyConfirmLayer.new()
                self.pushLayer:setItemIdToBuy(4001,"ui/buy/huoli_6.png",function()
                    end)
                self:addChild(self.pushLayer)
            end

            player.config.isBulletInfinite = true
            --self.bulletsButton:setEnabled(false)


        end,
    })

    self.bulletsMask = CCProgressTimer:create(CCSprite:create("ui/gameui/fire_cold_mask.png"))
    self.bulletsMask:setPosition(ccp(36,38))
    self.bulletsMask:setAnchorPoint(ccp(0.5,0.5))
    self.bulletsMask:setPercentage(0)
    self.bulletsMask:setType(kCCProgressTimerTypeRadial)
    self.bulletsButton:addChild(self.bulletsMask)

    self.bulletsLabel = ui.newAtlasLabel({
        text = string.format(":%02d",EquipSystem:getPropNum(4001)),
        start = '0',
        image = "ui/gameui/number_bar_bullet.png",
        align = ui.TEXT_ALIGN_LEFT,
        w = 12,
        h = 17,
        x = 18,
        y = -15,
    })
    self.bulletsButton:addChild(self.bulletsLabel)

    self.pauseButton = ui.newImageMenuItem({
        image = "ui/gameui/ui_pause_n.png",
        imageSelected = "ui/gameui/ui_pause_h.png",
        imageDisabled = "ui/gameui/ui_pause_h.png",
        x = display.left + 35,
        y = display.top - 35,
        listener = function()
            audio.playEffect(R.SOUND_BUTTON)
            self:performWithDelay(function()
                self:pauseGame()
            end, 0.02)
        end})


    self.menu = ui.newMenu({
        self.no2Button,self.bulletsButton,self.pauseButton
    })
    self:addChild(self.menu)

    self.bulletsButton:setEnabled(false);
    self.bulletsButton:setVisible(false);

    self.no2Button:setEnabled(false);
    self.no2Button:setVisible(false);
end


function ControlLayer:setEventHandler()
    self._eventHandler = function(event,args)
        self:eventHandler(event,args)
    end

    for k,eventName in pairs(ListenEvents) do
        DataCenter:addEventListener(eventName,self._eventHandler)
    end
end

function ControlLayer:removeEventHandler()
    for k,eventName in pairs(ListenEvents) do
        DataCenter:removeEventListener(eventName,self._eventHandler)
    end
end

--[[
游戏事件处理
]]
function ControlLayer:eventHandler(event,args)
    --游戏开始
    if event.name == "GAME_START" then
        audio.playEffect(R.SOUND_MOTO_START, false)

        GameSystem.gameState = GAME_STATE.START
        self:updatePlayerHP()

        self:performWithDelay(function()
            local objectsManager = Global:get("objectsManager")
            objectsManager.noEnemyFlag = false
        end, 3.0);
    --玩家死亡
    elseif event.name == "PLAYER_DEAD" then
        self:playerDead(event)
        --玩家复活
    elseif event.name == "PLAYER_REBORN" then
        self:playerReborn(event)
        --玩家受到攻击
    elseif event.name == "PLAYER_ATTACKED" then
        self:updatePlayerHP()
        self.record.numberOfHittings =  self.record.numberOfHittings + 1
        --游戏结束
    elseif event.name == "GAME_WIN" or event.name == "GAME_FAILED" then
        if event.name == GAME_FAILED then
            GameSystem.gameState = GAME_STATE.FAILED
        else
            GameSystem.gameState = GAME_STATE.WIN
        end

        local player = Global:get("player")
        GameSystem._record.lastGameInfo = clone(self.record)
        MyApp:gotoRank({})
        --跳跃动作
    elseif event.name == "PLAYER_FLY" then
        audio.playEffect(R.SOUND_MOTO_JUMP,false)
        self.record.numberOfFly =  self.record.numberOfFly + 1
        --杀死怪物
    elseif event.name == "MONSTER_DEAD" then
        local player = Global:get("player")

        if event.isBoss then
            if event.BossType == 1 then
                self.record.numberOfkillBoss =  self.record.numberOfkillBoss + 1
            elseif event.BossType == 2 then
                self.record.numberOfkillAdvBoss =  self.record.numberOfkillAdvBoss + 1
            else
                self.record.numberOfkillSuperBoss =  self.record.numberOfkillSuperBoss + 1
            end
        else
            self.record.numberOfkillEnemy =  self.record.numberOfkillEnemy + 1
        end

        if player:isCombo() then
            self:updateComboNumber()
        end

        self:updateMonsterNumber()

    elseif event.name == "CAR_DISTORYED" then
        self.record.numberOfhitCars = self.record.numberOfhitCars + 1
    elseif event.name == "TASK_FINISHED" then
        self:onTaskFinished(event)
    end
end

--[[
检查任务
--]]
function ControlLayer:checkTasks()
    GameSystem._record.lastGameInfo = self.record
    TaskSystem:checkAll();
end

--[[
任务完成的回调
--]]
function ControlLayer:onTaskFinished(event)
    local player = Global:get("player")
    if event.task then
        EffectHelper.floatAimInfo("(" .. TaskSystem:getFinishTaskCount() .. "/3) 完成 -【".. event.task.name .. "】")
        local stepId = GameSystem:getGuideId()
        if self.isGuide then
            return
        end

        if stepId == kGUIDE_TIP and TaskSystem:getFinishTaskCount() == 1 then
            --暂时无敌状态
            player.config.hasCrashBarrier = true
            self.isGuide = true
            self:performWithDelay(function()
                player.config.hasCrashBarrier = false

                CCDirector:sharedDirector():pause()
                self.guideLayer = CCGuideLayer:create();
                local pos = ccp(display.right - 285 , display.bottom + 110)
                local size = CCSize(280,60)

                self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.right - 180 , display.bottom + 300)
                self.guideLayer:addChild(self.guideText,100)

                self.arrowPointer = display.newSprite("ui/guide/arrow_big_down.png", display.right - 160 , display.bottom + 230)
                self.guideLayer:addChild(self.arrowPointer,100)

                self.guideLayer:show(
                    CCRectMake(pos.x ,pos.y, size.width, size.height),
                    function()
                        EffectHelper:cleanupfloatAimInfo()
                        self.isGuide = false
                        CCDirector:sharedDirector():resume()
                        GameSystem:finishGuide(stepId)
                        self.guideLayer:removeFromParentAndCleanup(true)
                        self:checkGuide()
                    end,
                    function()

                    end,true)
                self:addChild(self.guideLayer,999)
            end,0.2)
        end
    end
end

--[[
设置码表
]]
function ControlLayer:updateMoveDistance()
    local player = Global:get("player")
    if  self.distanceLabel then
        self.distanceLabel:setString(string.format("%05d",self.record.distance))
    end
end

--[[
更新仪表盘指针
]]
function ControlLayer:updateDashBoard()
    local player = Global:get("player")

    local speed = player.config.currentSpeed
    if speed > 120 then
        speed = 120
    end

    if player.config.currentSpeed == player.car.maxSpeed then
        self:startSpeedUpToolGuide()
    end

    if self.dashboardPin then
        local angle = speed / 160.0 * 360 - 45
        local oldAngle = self.dashboardPin:getRotation();

        local roationAction = CCRotateTo:create(math.abs(angle - oldAngle) / 200.0, angle)
        roationAction:setTag(99)
        self.dashboardPin:stopActionByTag(99)
        self.dashboardPin:runAction(roationAction)
    end

    self.record.maxNumberOfSpeed = math.max(self.record.maxNumberOfSpeed,speed)

    if not player.config.isDead then
        local posX = display.left + 140 - speed
        local x,y = player.avstar:getPosition()

        local moveAction = CCMoveTo:create(math.abs(posX - x) / 80.0,ccp(posX,y))
        moveAction:setTag(99)
        player.avstar:stopActionByTag(99)
        player.avstar:runAction(moveAction)
    end
end

--[[
更新玩家血条
--]]
function ControlLayer:updatePlayerHP()
    local player = Global:get("player")
    if self.gameHP then
        self.gameHP:stopAllActions()
        self.gameHP:runAction(CCProgressFromTo:create(0.5, self.gameHP:getPercentage(),(player.config.hp / player.config.maxHp) * 100))
    end

end

--[[
玩家死亡
--]]
function ControlLayer:playerDead(event)
    self:updatePlayerHP()
    self:updateDashBoard()

    local objectsManager = Global:get("objectsManager")
    objectsManager:idleAllEnemeys()

    self.record.numberOfDeads = self.record.numberOfDeads + 1

    self:performWithDelay(function()
        local stepId = GameSystem:getGuideId()
        if stepId <= kGUIDE_RESUME then
            self:playerReborn(event)
        else
            GameSystem.gameState = GAME_STATE.FAILED
            GameSystem._record.lastGameInfo = clone(self.record)
            local contineLayer = ContinueLayer:new()
            self:addChild(contineLayer)
        end
    end, 2)

    self:performWithDelay(function()
        audio.playEffect(R.SOUND_HIT_DEAD,false)
    end, 1)

    audio.playEffect(R.SOUND_HIT_CAR,false)
end

--[[
玩家复活
--]]
function ControlLayer:playerReborn(event)
    GameSystem.gameState = GAME_STATE.PRESTART
    local player = Global:get("player")
    local objectsManager = Global:get("objectsManager")

    objectsManager:killAllEnemys()

    player:doEvent("reborn")
    -- player:playAction("flydown")
    CCLuaLog("PLAYER_REBORN")
    self:updatePlayerHP()
end

--[[
暂停游戏
]]
function ControlLayer:pauseGame()
    GameSystem._record.lastGameInfo = clone(self.record)
    local pauseLayer = PauseLayer:new()
    self:addChild(pauseLayer);
end

--[[
更新子弹数字
]]
function ControlLayer:updateBulletsNumber()
    local player = Global:get("player")
    if self.buttletsLabel then
        local num = string.format(":%02d",player.gun.currentBullets)
        if num ~= self.buttletsLabel:getString() then
            self.buttletsLabel:setString(num)
            if player.gun.currentBullets == 0 then

                self.coldProgress:runAction(
                    CCSequence:createWithTwoActions(
                        CCProgressFromTo:create(player.gun.fillSpeed, 100,0),
                        CCCallFunc:create(function()
                            CCLuaLog("reload finish")

                            self.record.reloadTimes =  self.record.reloadTimes + 1
                            if self.record.reloadTimes == 3 then
                                self:startFireToolGuide()
                            end

                        end)))
            end
        end
    end
end

--[[
更新Monster击杀数字
]]
function ControlLayer:updateMonsterNumber()
    if self.monsterLabel then
        self.monsterLabel:setString(string.format("%05d",self.record.numberOfkillEnemy))
    end
end

--[[
更新ComBo数量显示
]]
function ControlLayer:updateComboNumber()
    local player = Global:get("player")
    self.comboBG:stopAllActions()
    self.comboNum:setString(string.format("%d",player.config.currentCombo))
    self.comboBG:setVisible(true)
    EffectHelper.playBubbleEffect(self.comboBG,function()
        self.comboBG:performWithDelay(function()
            self.comboBG:setVisible(false)
        end, 1)
    end)
    self.record.maxNumberOfCombs = math.max(self.record.maxNumberOfCombs,player.config.currentCombo)
end

function ControlLayer:updatePropNumber()
    self.no2Label:setString(string.format(":%02d",EquipSystem:getPropNum(4003)))
    self.bulletsLabel:setString(string.format(":%02d",EquipSystem:getPropNum(4001)))
end

--[[
注册触摸事件
]]
function ControlLayer:regsiterTouchEvent( ... )
    self:setTouchEnabled(true)
    self:setTouchMode(kCCTouchesAllAtOnce)
    self:addTouchEventListener(function(event,touches)
        x = touches[1]
        y = touches[2]
        if event == "began" then

            touchModeMoved(self,event,x,y)
            return cc.TOUCH_BEGAN
        end
        if event == "moved" then
            touchModeMoved(self,event,x,y)
            return cc.TOUCH_MOVED
        elseif event == "ended" then
            touchModeMoved(self,event,x,y)
        end
    end ,true, -127,false)
end

--[[
游戏逻辑判断
]]
function touchModeMoved(self,event,x,y)

    local player = Global:get("player")
    local mapLayer = Global:get("mapLayer")
    if event == "began" then
        self.lastPosY = y

    elseif event == "moved" then
        -- body
        if player.config.currentSpeed > 0 then
            if x < display.cx then
                self.lastPosY = self.lastPosY or 0
                local offsetY = y - self.lastPosY
                if offsetY > 0  then
                    player:moveAction("moveup");
                elseif offsetY < 0 then
                    player:moveAction("movedown");
                end
            end
            self.lastPosY = y
        end
    elseif event == "ended" then
        mapLayer:fixPlayerPosition()
    end
end


--[[
处理加速射击
]]
function ControlLayer:onEnterFrame(dt)

    local player = Global:get("player")
    local mapLayer = Global:get("mapLayer")

    --加速按钮
    if self.speedButton:getIsActive() then
        player.config.isSpeedUp = true
        player:doEvent("speedup");
    else
        player.config.isSpeedUp = false
        if player:isState("speed") then
            -- CCLuaLog("speeddown")
            player:doEvent("speeddown")
        end
    end

    -- --移动按钮
    -- if self.upButton:getIsActive() then
    --      player:moveAction("moveup");
    -- elseif self.downButton:getIsActive() then
    --      player:moveAction("movedown");
    -- else
    --      mapLayer:fixPlayerPosition()
    -- end

    --开火按钮
    if self.shootButton:getIsActive() then
        player.config.fireHold = true
        if player:canFire() then
            player:doEvent("shoot");
        end
    else
        player.config.fireHold = false
    end

end


function ControlLayer:onEnter()
    local mapLayer = Global:get("mapLayer")
    local player = Global:get("player")

    player:doEvent("startup")
    self:regsiterTouchEvent()

    self:setEventHandler()
    handler = scheduler.scheduleGlobal(function (dt)
        if GameSystem.gameState == GAME_STATE.START then
            self.record.startTime = self.record.startTime + dt
            self.record.maxTimeOfForeverMode = self.record.maxTimeOfForeverMode + dt
            self:onEnterFrame()
        end

        self:updateBulletsNumber()
    end, 0.05)

    DataCenter:dispatchEvent({name = "GAME_START"})

    self:checkGuide()
end


function ControlLayer:onExit()

    if handler then
        scheduler.unscheduleGlobal(handler);
    end
    self:removeEventHandler()
    --self:removeTouchEventListener()
end

--[[
引导
--]]
function ControlLayer:checkGuide()
    local player = Global:get("player")
    if player.config.isDead then
        return
    end

    local stepId = GameSystem:getGuideId()

    if stepId > kGUIDE_RESUME then
        self.bulletsButton:setEnabled(true);
        self.bulletsButton:setVisible(true);

        self.no2Button:setEnabled(true);
        self.no2Button:setVisible(true);
    end


    if stepId == kGUIDE_MOVE then
        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.left + 0 , 0)
        local size = CCSize(400,480)

        self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", 300 , 140)
        self.guideLayer:addChild(self.guideText,100)

        self.arrowPointer = display.newSprite("ui/guide/arrow_updown.png", 120 ,  display.cy)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.guideFinger = display.newSprite("ui/guide/guide_finger.png", 120 , display.cy - 60)
        self.guideLayer:addChild(self.guideFinger,100)

        local action1 = CCMoveTo:create(0.8, ccp(140 , display.cy + 30))
        local action2 = CCMoveTo:create(0.8, ccp(140 , display.cy - 60))
        self.guideFinger:runAction(CCRepeatForever:create(CCSequence:createWithTwoActions(action1,action2)))

        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()
                if not self.moveTime then
                    self.moveTime = 1
                    self:performWithDelay(function()
                        self.guideLayer:setVisible(false)
                    end, 0.2)
                end

                if self.moveTime >= 2 then
                    GameSystem:finishGuide(stepId)
                    self.guideLayer:removeFromParentAndCleanup(true)
                    self:checkGuide()
                end

                self.moveTime = self.moveTime + 1

            end,
            function()
                CCLuaLog("failed");
            end,false)
        self:addChild(self.guideLayer,999)
    elseif stepId == kGUIDE_FIRE then
        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.right - 105 , display.bottom + 15)
        local size = CCSize(90,90)

        self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.right - 70 , display.bottom + 230)
        self.guideLayer:addChild(self.guideText,100)

        self.arrowPointer = display.newSprite("ui/guide/arrow_big_down.png", display.right - 60 , display.bottom + 150)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()

            end,
            function()
                CCLuaLog("failed");
            end,true)
        self:addChild(self.guideLayer,999)
    elseif stepId == kGUIDE_SPEED then
        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.right - 205 , display.bottom + 15)
        local size = CCSize(90,90)

        self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.right - 170 , display.bottom + 230)
        self.guideLayer:addChild(self.guideText,100)

        self.arrowPointer = display.newSprite("ui/guide/arrow_big_down.png", display.right - 160 , display.bottom + 150)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()

            end,
            function()
            -- CCLuaLog("failed");
            end,true)
        self:addChild(self.guideLayer,999)
    elseif stepId == kGUIDE_PAUSE then

        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.left + 8 , display.top - 60)
        local size = CCSize(56,56)

        self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.left + 170 , display.top - 100)
        self.guideLayer:addChild(self.guideText,100)

        self.arrowPointer = display.newSprite("ui/guide/arrow_small_left.png", display.left + 120 , display.top  - 30)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()
                display.resume()
                GameSystem:finishGuide(stepId)
                self.guideLayer:removeFromParentAndCleanup(true)
                self:checkGuide()
            end,
            function()
            -- CCLuaLog("failed");
            end,true)
        self:addChild(self.guideLayer,999)
        display.pause()

    elseif stepId == kGUIDE_TASK then
        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.cx - 130 , display.cy - 110)
        local size = CCSize(265,100)

        self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.cx , display.bottom + 35)
        self.guideLayer:addChild(self.guideText,100)

        self.arrowPointer = display.newSprite("ui/guide/arrow_small_up.png", display.cx , display.bottom + 100)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()
                GameSystem:finishGuide(stepId)
                self.guideLayer:removeFromParentAndCleanup(true)
                self:checkGuide()
            end,
            function()
                GameSystem:finishGuide(stepId)
                self.guideLayer:removeFromParentAndCleanup(true)
                self:checkGuide()
            end,true)
        self:addChild(self.guideLayer,999)
    elseif stepId == kGUIDE_TITLE then
        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.cx - 70 , display.cy - 10)
        local size = CCSize(140,100)

        self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.cx + 250, display.cy + 40)
        self.guideLayer:addChild(self.guideText,100)

        self.arrowPointer = display.newSprite("ui/guide/arrow_small_left.png", display.cx + 100, display.cy + 40)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()
                GameSystem:finishGuide(stepId)
                self.guideLayer:removeFromParentAndCleanup(true)
                self:checkGuide()
            end,
            function()
                GameSystem:finishGuide(stepId)
                self.guideLayer:removeFromParentAndCleanup(true)
                self:checkGuide()
            end,true)
        self:addChild(self.guideLayer,999)
    elseif stepId == kGUIDE_RESUME then

        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.right - 220 , display.bottom + 10)
        local size = CCSize(200,80)

        self.arrowPointer = display.newSprite("ui/guide/arrow_big_down.png", display.right - 120, display.bottom + 140)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()
                GameSystem:finishGuide(stepId)
                self.guideLayer:removeFromParentAndCleanup(true)
                self.record.reloadTimes = 0
                self:checkGuide()
            end,
            function()

            end,true)
        self:addChild(self.guideLayer,999)

    end
end

function ControlLayer:startFireToolGuide()
    if self.inGuide then
        return
    end

    local player = Global:get("player")
    if player.config.isDead then
        return
    end

    local stepId = GameSystem:getGuideId()

    if stepId == kGUIDE_BULLETS then

        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.right - 380 , display.bottom + 25)
        local size = CCSize(75,75)

        self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.right - 350, display.bottom + 200)
        self.guideLayer:addChild(self.guideText,100)

        self.arrowPointer = display.newSprite("ui/guide/arrow_small_down.png", display.right - 350, display.bottom + 140)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.inGuide = true
        display.pause()
        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()
                display.resume()
                GameSystem:finishGuide(stepId)
                self.guideLayer:removeFromParentAndCleanup(true)

                self:performWithDelay(function()
                    display.pause()
                    stepId = GameSystem:getGuideId()
                    self.guideLayer = CCGuideLayer:create();
                    --kGUIDE_BULLETS_NEXT
                    local pos = ccp(display.right - 210 , display.top - 80)
                    local size = CCSize(100,40)

                    self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.right - 300, display.cy)
                    self.guideLayer:addChild(self.guideText,100)

                    self.arrowPointer = display.newSprite("ui/guide/arrow_small_up.png", display.right - 200, display.cy + 80)
                    self.guideLayer:addChild(self.arrowPointer,100)

                    self.guideLayer:show(
                        CCRectMake(pos.x ,pos.y, size.width, size.height),
                        function()
                            self.inGuide = false
                            GameSystem:finishGuide(stepId)
                            self.guideLayer:removeFromParentAndCleanup(true)
                            self:checkGuide()
                            display.resume()
                        end,
                        function()
                            self.inGuide = false
                            GameSystem:finishGuide(stepId)
                            self.guideLayer:removeFromParentAndCleanup(true)
                            self:checkGuide()
                            display.resume()
                        end,true)
                    self:addChild(self.guideLayer,999)
                end, 0.1)


            end,
            function()

            end,true)
        self:addChild(self.guideLayer,999)

    end

end


function ControlLayer:startSpeedUpToolGuide()
    if self.inGuide then
        return
    end

    local player = Global:get("player")
    if player.config.isDead then
        return
    end

    local stepId = GameSystem:getGuideId()

    if stepId == kGUIDE_SPEEDUP then
        self.guideLayer = CCGuideLayer:create();
        local pos = ccp(display.right - 300 , display.bottom + 25)
        local size = CCSize(75,75)

        self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.right - 275, display.bottom + 200)
        self.guideLayer:addChild(self.guideText,100)

        self.arrowPointer = display.newSprite("ui/guide/arrow_small_down.png", display.right - 275, display.bottom + 140)
        self.guideLayer:addChild(self.arrowPointer,100)

        self.inGuide = true
        display.pause()
        self.guideLayer:show(
            CCRectMake(pos.x ,pos.y, size.width, size.height),
            function()
                GameSystem:finishGuide(stepId)

                self.guideLayer:removeFromParentAndCleanup(true)

                --kGUIDE_SPEEDUP_NEXT
                display.resume()
                stepId = GameSystem:getGuideId()
                self.guideLayer = CCGuideLayer:create();

                local pos = ccp(display.right - 125 , display.top - 130)
                local size = CCSize(120,120)

                self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", display.right - 260, display.cy)
                self.guideLayer:addChild(self.guideText,100)

                self.arrowPointer = display.newSprite("ui/guide/arrow_small_up.png", display.right - 80, display.cy + 60)
                self.guideLayer:addChild(self.arrowPointer,100)

                self.guideLayer:show(
                    CCRectMake(pos.x ,pos.y, size.width, size.height),
                    function()
                        self.inGuide = false
                        GameSystem:finishGuide(stepId)
                        self.guideLayer:removeFromParentAndCleanup(true)
                        self:checkGuide()
                    end,
                    function()
                        self.inGuide = false
                        GameSystem:finishGuide(stepId)
                        self.guideLayer:removeFromParentAndCleanup(true)
                        self:checkGuide()
                    end,true)
                self:addChild(self.guideLayer,999)
            end,
            function()

            end,true)
        self:addChild(self.guideLayer,999)
    end
end


return ControlLayer