--*******************************************
-- Desc: 地图类
-- File: MapLayer.lua    
-- Author: zhouhaifeng
-- Date: 2014-03-09 15:44:01
--*******************************************

local LevelData = import("..datas.LevelData")
local MapData = import("..datas.MapData")
local ObjectsManager = import("..sprites.ObjectsManager")

local MapLayer = class("MapLayer",function()
    return display.newLayer("MapLayer")
end)

--[[
    初始化
    params: 地图信息
]]
function MapLayer:ctor()
   -- 设置移动速度  

    local params = LevelData[LevelSystem:getCurrentPlayMap()]
    self:loadMapData(params);

    Global:set("objectsManager",ObjectsManager)
    ObjectsManager:reset()

    self.monsters = params.monsters
    self.timeCount = 0

    return self;
end

--[[
    设置移动速度
]]
function MapLayer:setPlayerSpeed(speed)
    self.speed = speed
    
    if self.speed then
        self.skyVelocity = -(self.speed / 10 ) * 60 
        self.tileMapVelocity = -(self.speed / 6) * 60 
    end
end

--[[
    加载地图信息
]]
function MapLayer:loadMapData(params)
    if type(params) ~= "table" then
        CCLuaLog("[LUA ERROR] mapdata params need a table value")
        return 
    end
    self.tileMap = CCTMXTiledMap:create(params.path)  
    display.setIsBackgroundImage(self.tileMap)
    self.tileMap:setAnchorPoint(ccp(0,0))     
    self:addChild(self.tileMap, 0, 0) 

    -- 设置高度  
    self.skyWidth= self.tileMap:getMapSize().width * self.tileMap:getTileSize().width  
    self.tileMapWidth = self.tileMap:getMapSize().width * self.tileMap:getTileSize().width  
    local offsetX = self.tileMap:getPositionX() + self.tileMapWidth;
    --设置重复偏移
    for layerName,layerProp in pairs(MapData.repeatLayers) do
        local layer = self.tileMap:layerNamed(layerName)
        if layer then
            layer:setPosition(ccp(offsetX,0)) 
        end
    end
    --加载远景层
    self.skysList = {}
    for layerIndex,layerName in pairs(MapData.skyLayers) do
        self.skysList[layerIndex] = self.tileMap:layerNamed(layerName) 
    end

    --加载地图层
    self.mapsList = {}
    for layerIndex,layerName in pairs(MapData.mapLayers) do
        self.mapsList[layerIndex] = self.tileMap:layerNamed(layerName) 
    end

    self:setAllAliasTexParameters()
    self:fixLayerZorder()

    if DEBUG_LABEL then
       self.gridLayer = GridLayer.new()
       self.tileMap:addChild(self.gridLayer,2000)
    end
   
end

function MapLayer:loadMonsterData()
    local controlLayer = Global:get("controlLayer")
    ObjectsManager:randomMakeMap(controlLayer.record.distance)
end


--[[
    设置tmx地图属性
]]
function MapLayer:setAllAliasTexParameters()

    local mapObjects = self.tileMap:getChildren();
    -- 遍历所有子对象
    for i = 0, mapObjects:count()-1 do
        local child = tolua.cast(mapObjects:objectAtIndex(i), "CCNode")
        child:getTexture():setAntiAliasTexParameters()
        child:setZOrder(child:getZOrder() * 50)
        -- CCLuaLog( " name:" .. child:getLayerName() .. " zorder:" .. child:getZOrder());
    end
end

--[[
    调整Zorder
]]
function MapLayer:fixLayerZorder()
    local zorder = self.tileMap:layerNamed("lm_0"):getZOrder()
    
    self.minZorder =  self.tileMap:layerNamed("lm_0"):getZOrder() + 2
    self.maxZorder =  self.tileMap:layerNamed("jg_1"):getZOrder() - 2
    
end

--[[
    添加player
]]
function MapLayer:addPlayerToMap(player)
    self.avstar = player;
    player:setAnchorPoint(ccp(0,0))
    self.tileMap:addChild(player)

    local playerX = display.left + 120
    local playerY = display.cy
    self.avstar:setPosition(playerX,playerY)
    self:fixPlayerPosition()
end


--[[
    设置主角的位置，Zorder
]]
function MapLayer:setPlayerPosition(event,x,y)
    if ObjectsManager:canPlayerMoveTo(x, y) then    
        self:setObjectPosition(self.avstar,x,y)
    else 
        CCLuaLog("can not move (" .. x .. "," .. y ..")")
        self:fixPlayerPosition()
    end
end

function MapLayer:fixPlayerPosition()
    -- setZorder
    local x,y = self.avstar:getPosition()
    local pos = ccp(x,y)
    local col = math.floor((pos.y - 64) / 32) 
   
    if col <= 0 then
        col = 0
    elseif col >= 8 then
        col = 8
    end

    local newZ = self.maxZorder
    newZ = newZ - col*2 + 1
    -- pos.y = col * 32 + 64 + 10   --中心位置
    if y >= 80 and y <= 304 then
        self.avstar:setPosition(pos)  
        self.tileMap:reorderChild(self.avstar, newZ)
        self.avstar.col = col 
    end
    -- if DEBUG_LABEL then
    --      self.avstar.nameLabel:setString(self.avstar:getZOrder())   
    -- end
end

--[[
    设置物体在地图中的位置，Zorder
]]
function MapLayer:setObjectPosition(object,x,y)
    local pos = ccp(x,y)
    
    if y >= 80 and y <= 304 then
        --setZorder
        local col = math.floor((pos.y - 64) / 32)
        local newZ = self.maxZorder
        if object == self.avstar then
            newZ = newZ - col * 2 + 1
            self.avstar.col = col
            self.avstar:setPosition(pos)  
            self.tileMap:reorderChild(self.avstar, newZ)
        else
           newZ = newZ - col * 2 + 0
           object.col = col
           object:setPosition(pos)  
            local move = CCMoveTo:create(0.05, pos)
            object:runAction(move)
            self.tileMap:reorderChild(object, newZ)
        end
    end
end


function MapLayer:onEnterFrame(dt)
    local controlLayer = Global:get("controlLayer")
    local player = Global:get("player")
    self:setPlayerSpeed(player.config.currentSpeed); 

    self.timeCount = self.timeCount + 1
    if  self.timeCount == 4 then
        controlLayer:updateDashBoard()
        self.timeCount = 0
    end

    if self.speed then
        if self.speed > 0 then
            local skyDistance = math.abs(math.floor(self.skyVelocity * dt))
            local moveDistance = math.abs(math.floor(self.tileMapVelocity * dt))

            --更新记录数据
            local offset = moveDistance / 60
            controlLayer.record.speedOfDistance = controlLayer.record.speedOfDistance + offset * DISTANCE_FACTOR
            controlLayer.record.normalOfDistance = controlLayer.record.normalOfDistance + offset * DISTANCE_FACTOR
            controlLayer.record.distance = controlLayer.record.distance + offset * DISTANCE_FACTOR

            controlLayer:updateMoveDistance()
    
            --地图层
            for key,mapNode in pairs(self.mapsList) do 
                mapNode:setPositionX(mapNode:getPositionX() - moveDistance);  
                if mapNode:getPositionX() <= -self.tileMapWidth then   
                    mapNode:setPositionX(mapNode:getPositionX() + self.tileMapWidth * 2 );  
                end  
            end
            --更新怪物移动
            ObjectsManager:updateFrame(dt, moveDistance)

            --远景层
            for key,skyNode in pairs(self.skysList) do
                skyNode:setPositionX(skyNode:getPositionX() - skyDistance);  
                if skyNode:getPositionX() <= -self.skyWidth then   
                    skyNode:setPositionX(skyNode:getPositionX() + self.skyWidth * 2 );  
                end
            end
            
            --产生怪物
            self:loadMonsterData()
        end
    end
end

function MapLayer:onEnter()
    self:scheduleUpdate(function(dt) 
        local state = GameSystem.gameState
        if  state == GAME_STATE.START or state == GAME_STATE.PRESTART then
            self:onEnterFrame(dt) 
        end
    end)
end

function MapLayer:onExit()
    self:unscheduleUpdate()
end

return MapLayer