--*********************************************
-- Desc: 装备选择界面
-- File: EquipmentScene.lua
-- Author: zhouhaifeng
-- Date: 2014-06-20 11:27:10
--*********************************************

local GameItems = import("app.datas.GameItems")
local CoverFlow = import("app.utils.ui.CoverFlow")
local BuyConfirmLayer = import("app.layers.BuyConfirmLayer")
local PlayerInfoLayer = import("..layers.PlayerInfoLayer")

local EquipmentScene = class("EquipmentScene", function()
    return display.newScene("EquipmentScene")
end)

function EquipmentScene:ctor()

    self.bgLayer = display.newSprite("ui/start/ui_bg.jpg", display.cx, display.cy)
    display.setIsBackgroundImage(self.bgLayer)
    self:addChild(self.bgLayer)

	self.playerInfoLayer = PlayerInfoLayer.new()
	self:addChild(self.playerInfoLayer,10)

    local rect = CCRect(-20,-20,160,300)
    local width = display.width / 4


    --武器
    self.coverflow2 = CoverFlow.new(rect,1)
    self.coverflow2:setCellEventHandler(function( ... )
        self:itemEventHandler(...)
    end) 
    self.coverflow2:setTag(kItemTypeWeapon)

    if GameSystem:getGuideId() == kGUIDE_EQUIP then
       local weaponData = EquipSystem:getItemTypeTable(kItemTypeWeapon)
       table.insert(weaponData, 1, GameItems[2000])
       self.coverflow2:setDataSource(weaponData)
    else 
        local weaponData = EquipSystem:getItemTypeTable(kItemTypeWeapon)
        self.coverflow2:setDataSource(weaponData)
    end
    self.coverflow2:setPosition(90, display.cy - 92)
    self:addChild(self.coverflow2) 

    --车
    self.coverflow1 = CoverFlow.new(rect,1) 
    self.coverflow1:setCellEventHandler(function( ... )
        self:itemEventHandler(...)
    end)
    self.coverflow1:setTag(kItemTypeCars)
    if GameSystem:getGuideId() == kGUIDE_EQUIP or GameSystem:getGuideId() == kGUIDE_CAR then
       local carsData = EquipSystem:getItemTypeTable(kItemTypeCars)
       table.insert(carsData, 1, GameItems[1000])
       self.coverflow1:setDataSource(carsData)
    else 
        local carsData = EquipSystem:getItemTypeTable(kItemTypeCars)
        self.coverflow1:setDataSource(carsData)
    end
    self.coverflow1:setPosition(360, display.cy - 92)
    self:addChild(self.coverflow1) 

    --衣服
    self.coverflow3 = CoverFlow.new(rect,1) 

    self.coverflow3:setCellEventHandler(function( ... )
        self:itemEventHandler(...)
    end)
    self.coverflow3:setTag(kItemTypeClothes)
    self.coverflow3:setDataSource(EquipSystem:getItemTypeTable(kItemTypeClothes))
    self.coverflow3:setPosition(625, display.cy - 92)
    self:addChild(self.coverflow3) 

    self.bg = display.newSprite("ui/level/ui_jiange.png", display.cx, display.cy)
    display.setIsBackgroundImage(self.bg)
    self:addChild(self.bg)

    self.layer = ccs.layer()
    self:addChild(self.layer)

    self.goLevelButton = ccs.button({
        normal  = "ui/common/btn_chooseLevel_n.png",
        pressed = "ui/common/btn_chooseLevel_h.png",
        x = display.right - 120,
        y = display.bottom + 40,
        listener = {[ccs.TouchEventType.ended] = function() 
            audio.playEffect(R.SOUND_BUTTON)
    		MyApp:goToForeverMode()
         end},
    })
    self.layer:addWidget(self.goLevelButton)

    self.backButton = ccs.button({
        normal = "ui/common/btn_back_n.png",
        pressed = "ui/common/btn_back_h.png",
        x = display.left + 120,
        y = display.bottom + 40,
        listener = {[ccs.TouchEventType.ended] = function()
        	    audio.playEffect(R.SOUND_BUTTON)
            	MyApp:goStart()
        end},
    })
    self.layer:addWidget(self.backButton) 

    self:initEquipList()

   

end

function EquipmentScene:initEquipList()

    self.coverflow1:moveToIndex(math.fmod(EquipSystem:getEquipItemWithType(kItemTypeCars),1000))
    self.coverflow2:moveToIndex(math.fmod(EquipSystem:getEquipItemWithType(kItemTypeWeapon),1000))
    self.coverflow3:moveToIndex(math.fmod(EquipSystem:getEquipItemWithType(kItemTypeClothes),1000))

end

function EquipmentScene:itemEventHandler(args)
    if args.event == "dataDidChange" then 
         -- CCLuaLog(args.coverflow:getTag() .. "  event:" .. args.event ..  "  value:" .. args.newValue )
        local cell = args.currentCell
        local flag = EquipSystem:checkIsExist(cell.itemId)
        if flag then
           EquipSystem:equipItem(cell.itemId)
        end

        local stepId = GameSystem:getGuideId()
        if stepId==kGUIDE_EQUIP then
            if cell.itemId== 2001 then
                 self:performWithDelay(function()
                            GameSystem:finishGuide(stepId)
                            self.guideLayer:removeFromParentAndCleanup(true)
                            self:checkGuide() 
                  end
                 , 0.4)
            end
        elseif stepId==kGUIDE_CAR then
            if cell.itemId== 1001 then
                 self:performWithDelay(function()
                            GameSystem:finishGuide(stepId)
                            self.guideLayer:removeFromParentAndCleanup(true)
                            self:checkGuide() 
                  end
                 , 0.4)
            end
        end
       

    elseif args.event == "onTaped" then 
        local cell = args.currentCell

        local flag = EquipSystem:checkIsExist(cell.itemId)
        if flag then
            cell.lock:setVisible(false)
        else
            self:buyItem(cell)   
        end
        
    elseif args.event == "setupCells" then
        local cell = args.currentCell
        local flag = EquipSystem:checkIsExist(cell.itemId)
        if flag then
            cell.lock:setVisible(false)
        else
            cell.lock:setVisible(true)
        end
    end

end

function EquipmentScene:buyItem(cell) 
    local itemId = cell.itemId
    local buyConfirmLayer = BuyConfirmLayer.new()
    buyConfirmLayer:setItemIdToBuy(itemId,"ui/level/ui_item_" .. itemId ..".png",function(state,prop)
        local flag = EquipSystem:checkIsExist(cell.itemId)
        if flag then
            cell.lock:setVisible(false)
        else
            cell.lock:setVisible(true)
        end
    end)
    self:addChild(buyConfirmLayer,100)
end

function EquipmentScene:onEnter()
    CCDirector:sharedDirector():setProjection(kCCDirectorProjection3D)
    self:checkGuide();
end

function EquipmentScene:onExit()
        CCDirector:sharedDirector():setProjection(kCCDirectorProjection2D)
end

function EquipmentScene:checkGuide()
     local stepId = GameSystem:getGuideId()
     if stepId == kGUIDE_EQUIP then
            self.guideLayer = CCGuideLayer:create();

            local x,y = self.coverflow2:getPosition()
            local pos = ccp(x - 20,y + 50)
            local size = CCSize(160, 100)

            self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", pos.x + 250 , pos.y - 60)
            self.guideLayer:addChild(self.guideText,100)

            self.arrowPointer = display.newSprite("ui/guide/arrow_big_down.png", pos.x + 80 , pos.y - 60)
            self.guideLayer:addChild(self.arrowPointer,100)

            -- self.arrowPointer = display.newSprite("ui/guide/arrow_big_up.png", pos.x + 80 , pos.y + 150)
            -- self.guideLayer:addChild(self.arrowPointer,100)
      
            self.guideFinger = display.newSprite("ui/guide/guide_finger.png", pos.x + 160 , pos.y + 80)
            self.guideLayer:addChild(self.guideFinger,100)

            local action1 = CCMoveTo:create(0.8, ccp(pos.x + 160 , pos.y + 0))
            local action2 = CCMoveTo:create(0.1, ccp(pos.x + 160 , pos.y + 80))
            self.guideFinger:runAction(CCRepeatForever:create(CCSequence:createWithTwoActions(action1,action2)))

            self.guideLayer:show(
                CCRectMake(pos.x ,pos.y, size.width, size.height),
                function()
                    
                end,
                function()
                    CCLuaLog("failed");
                end,true)
            self:addChild(self.guideLayer,999)
    elseif stepId == kGUIDE_CAR then
            self.guideLayer = CCGuideLayer:create();

            local x,y = self.coverflow1:getPosition()
            local pos = ccp(x - 20,y + 50)
            local size = CCSize(160, 100)

            self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", pos.x + 250 , pos.y - 60)
            self.guideLayer:addChild(self.guideText,100)

            self.arrowPointer = display.newSprite("ui/guide/arrow_big_down.png", pos.x + 80 , pos.y - 60)
            self.guideLayer:addChild(self.arrowPointer,100)

            -- self.arrowPointer = display.newSprite("ui/guide/arrow_big_up.png", pos.x + 180 , pos.y + 150)
            -- self.guideLayer:addChild(self.arrowPointer,100)
      
            self.guideFinger = display.newSprite("ui/guide/guide_finger.png", pos.x + 160 , pos.y + 80)
            self.guideLayer:addChild(self.guideFinger,100)

            local action1 = CCMoveTo:create(0.8, ccp(pos.x + 160 , pos.y + 0))
            local action2 = CCMoveTo:create(0.1, ccp(pos.x + 160 , pos.y + 80))
            self.guideFinger:runAction(CCRepeatForever:create(CCSequence:createWithTwoActions(action1,action2)))

            self.guideLayer:show(
                CCRectMake(pos.x ,pos.y, size.width, size.height),
                function()
                    
                end,
                function()
                    CCLuaLog("failed");
                end,true)
            self:addChild(self.guideLayer,999)
    elseif stepId == kGUIDE_EQUIP_NEXT then
             self.guideLayer = CCGuideLayer:create();
             local pos = self.goLevelButton:getPosition()
             local size = self.goLevelButton:getSize()
         
             self.guideText = display.newSprite("ui/guide/guide_text_".. stepId ..".png", pos.x - 10 , pos.y + 150)
             self.guideLayer:addChild(self.guideText,100)

             self.arrowPointer = display.newSprite("ui/guide/arrow_big_down.png", pos.x - 10 , pos.y + 90)
             self.guideLayer:addChild(self.arrowPointer,100)

             self.guideLayer:show(
                CCRectMake(pos.x - size.width/2, pos.y-size.height/2, size.width, size.height),
                function()
                    GameSystem:finishGuide(stepId)
                end,
                function()
                    CCLuaLog("failed");
                end,true)
            self:addChild(self.guideLayer,999)
    end
end

return EquipmentScene
