--*******************************************
-- Desc: 游戏主体
-- File: GameScene.lua
-- Author: zhouhaifeng
-- Date: 2014-03-12 20:40:51
--*******************************************

local Player =  import("..sprites.Player")
local MapLayer = import("..layers.MapLayer")
local ControlLayer = import("..layers.ControlLayer")
local ActivateLayer = import("..layers.ActivateLayer")

local GameScene = class("GameScene", function()
    return display.newScene("GameScene")
end)

function GameScene:ctor()
    self.mapLayer = MapLayer.new()
    Global:set("mapLayer",self.mapLayer)
    self:addChild(self.mapLayer)

    self.player = Player.new()
    Global:set("player",self.player)
    self.player.avstar:setPosition(ccp(display.left+200,display.bottom + 100))
    self.mapLayer:addPlayerToMap(self.player.avstar)

    self.controlLayer = ControlLayer.new()
    Global:set("controlLayer",self.controlLayer)
    self:addChild(self.controlLayer);    

    audio.playBackgroundMusic("sound/bgm.ogg",true);
end

function GameScene:onEnter()
    g_IsInGame = true
end

function GameScene:onEnterTransitionFinish()
     if not LevelSystem:checkIsActivate() then
        local activateLayer = ActivateLayer:new()
        self:addChild(activateLayer)
    end
end

function GameScene:onExit()
    g_IsInGame = false

    Global:set("player",nil);
    Global:set("mapLayer",nil);
    Global:set("controlLayer",nil);
    
    
    CCLuaLog("GameScene:onExit()")
    local MonsterPools = Global:get("MonsterPools")

    if MonsterPools then
        MonsterPools:purge()
        Global:set("MonsterPools",nil)
    end

    local BlockPools = Global:get("BlockPools")
    if BlockPools then
        BlockPools:purge()
        Global:set("BlockPools",nil)
    end

    local BulletPools = Global:get("BulletPools")
    if BulletPools then
        BulletPools:purge()
        Global:set("BulletPools",nil)
    end
    
    GUIReader:purgeGUIReader()
    CCArmatureDataManager:purge()
    ActionManager:purgeActionManager();

    audio.playBackgroundMusic("sound/background.mp3",true)
    
end

return GameScene
