--*******************************************
-- Desc: 加载界面
-- File: LoadingScene.lua
-- Author: zhouhaifeng
-- Date: 2014-03-19 00:08:33
--*******************************************

local EffectManager = import("app.sprites.EffectManager")
local TaskInfoCell = import("app.items.TaskInfoCell")

local LoadingScene = class("LoadingScene", function()
    return display.newScene("LoadingScene")
end)

--资源类型
local PRE_RES_TYPE = {
	IMAGE_TEXTURE = 1,		--图片纹理
	SPRITE_FRAME = 2,		--动画帧
	OBJECT_POOLS = 3,		--对象池
	MUSIC_FILE = 4,			--背景音乐文件
	EFFECT_FILE = 5,		--音效文件
}

function LoadingScene:ctor()
	CCTextureCache:sharedTextureCache():removeAllTextures()

	--初始化特效
	EffectManager:init()

	self.preResTables = {

    }

    self.preFrameTables = {
    	
	} 

 	self.preObjectsPools = {
    	 {packageName = "app.sprites.MonsterPools", className="MonsterPools"},
    	 {packageName = "app.sprites.BlockPools", className="BlockPools"},
    	 {packageName = "app.sprites.BulletPools", className="BulletPools"},
	}


    local manager = CCArmatureDataManager:sharedArmatureDataManager()
    manager:addArmatureFileInfo("animations/effects/speedup.ExportJson")
    manager:addArmatureFileInfo("animations/effects/shockwave.ExportJson")
    

	self:initUI()    
end

function LoadingScene:initUI()
	self.loading = display.newSprite("loading/loading.jpg", display.cx, display.cy)
	display.setIsBackgroundImage(self.loading)
	self:addChild(self.loading)

	self.taskInfoCell = TaskInfoCell.new()
   	self.taskInfoCell:setup({ 
        max = 3,
        cur = 1,
        on = "ui/playerinfo/ui_stars_1.png",
        off = "ui/playerinfo/ui_stars_0.png",
        x = display.cx - 110,
        y = display.cy - 40,   
   	})

  	self:addChild(self.taskInfoCell)
  	self.taskInfoCell:setTaskInfo(TaskSystem._tasks)
end


function LoadingScene:onEnter()
	LoadingHelper:reset()
	self:performWithDelay(function()
		LoadingHelper:preLoading(PRE_RES_TYPE.IMAGE_TEXTURE , self.preResTables) 
		LoadingHelper:preLoading(PRE_RES_TYPE.OBJECT_POOLS , self.preObjectsPools) 
		LoadingHelper:preLoading(PRE_RES_TYPE.SPRITE_FRAME , self.preFrameTables)
		LoadingHelper:setStepCallbackHandler(function (cur,total)
	   		-- self.loadingbar:setPercent(math.floor(cur/total * 100))
	        if cur >= total then
	        	self:performWithDelay(function()
	        		MyApp:goToGame()
	        	end, 1.0)
	        end
		 end)
		LoadingHelper:start()
	end,2.0)
end

function LoadingScene:onExit()
	self.preResTables = nil
	self.preObjectsPools = nil
	self.preFrameTables = nil
end

return LoadingScene


