--***************************************
-- Desc: 结算得分层
-- File: ScoreScene.lua
-- Author: zhouhaifeng
-- Date: 2014-03-09 15:44:01
--***************************************

local PlayerInfoLayer = import("app.layers.PlayerInfoLayer")
local PlayerRecordBar = import("app.items.PlayerRecordBar")
local TaskInfoLayer = import("app.layers.TaskInfoLayer")
local BuyConfirmLayer = import("app.layers.BuyConfirmLayer")
local LevelData = import("..datas.LevelData")

local ScoreScene = class("ScoreScene",function()
	return display.newScene("ScoreScene")
end)

--[[
	初始化
	params: 地图信息
]]
function ScoreScene:ctor()
    local params = LevelData[LevelSystem:getCurrentPlayMap()]

    self.tileMap = CCTMXTiledMap:create(params.path)  
    display.setIsBackgroundImage(self.tileMap)
    self.tileMap:setAnchorPoint(ccp(0,0))
    self:addChild(self.tileMap, 0, 0)

    self:addChild(display.newColorLayer(ccc4(0,0,0,192)))

    if TaskSystem:getFinishTaskCount() >= 3 and LevelSystem._level.currentLevel < LevelSystem._level.maxLevel then
        self:performWithDelay(function()
            local taskInfo = TaskInfoLayer.new()
            self:addChild(taskInfo,100)

            taskInfo:setCallBack(function()
                 -- CCLuaLog("callback function")   
                 self:setupUI()
                 self:updateUI()
            end) 
        end,1.5);
    else
        self:setupUI()
        self:updateUI()
    end
end

function ScoreScene:setupUI()
    self.layer = ccs.layer()
    self:addChild(self.layer)

    self.playerInfoLayer = PlayerInfoLayer.new()
    self:addChild(self.playerInfoLayer)

    self.gotostoreButton = ccs.button({
        normal  = "ui/score/btn_equip_n.png",
        pressed = "ui/score/btn_equip_h.png",
        x = display.left + 120,
        y = display.bottom + 40,
        listener = {[ccs.TouchEventType.ended] = function() 
            MyApp:goToEquipment()
         end},
    })

    self.gotoNextButton = ccs.button({
        normal  = "ui/score/btn_restart_n.png",
        pressed = "ui/score/btn_restart_h.png",
        x = display.right - 120,
        y = display.bottom + 40,
        listener = {[ccs.TouchEventType.began] = function() 
            MyApp:goToForeverMode()
         end},
    })

    self.scoreBg = ccs.image({
        image  = "ui/score/ui_score_bg.png",
        x = display.cx,
        y = display.cy + 20,
    })
    self.layer:addWidget(self.scoreBg)


    self.scoreTitle = ccs.image({
        image  = "ui/score/score_title.png",
        x = display.cx,
        y = display.cy + 20,
    })
    self.layer:addWidget(self.scoreTitle)
    

    self.distanceBar = PlayerRecordBar.new("distance")
    self.distanceBar:setPosition(ccp(display.cx,display.cy + 50))
    self.layer:addWidget(self.distanceBar)

    self.killBar = PlayerRecordBar.new("kill")
    self.killBar:setPosition(ccp(display.cx,display.cy + 10))
    self.layer:addWidget(self.killBar)

    self.comboBar = PlayerRecordBar.new("combo")
    self.comboBar:setPosition(ccp(display.cx,display.cy - 30))
    self.layer:addWidget(self.comboBar)

    self.speedBar = PlayerRecordBar.new("speed")
    self.speedBar:setPosition(ccp(display.cx,display.cy - 70))
    self.layer:addWidget(self.speedBar)

    self.layer:addWidget(self.gotostoreButton)
    self.layer:addWidget(self.gotoNextButton)

    self.timeTitle = ccs.image({
        image  = "ui/score/ui_time_title.png",
        x = display.left + 60,
        y = display.bottom + 120,
    })
    self.layer:addWidget(self.timeTitle)

    self.moneyTitle = ccs.image({
        image  = "ui/score/ui_money_title.png",
        x = display.right - 240,
        y = display.bottom + 120,
    })
    self.layer:addWidget(self.moneyTitle)


    self.moneyIcon = ccs.image({
        image  = "ui/score/ui_money_icon.png",
        x = display.right - 180,
        y = display.bottom + 120,
    })
    self.layer:addWidget(self.moneyIcon)

    self.moneyLabel = ccs.labelAtlas({
        text = "0",
        start = '0',
        image = "ui/score/number_bar_money.png",
        w = 24,
        h = 30,
        align = ui.TEXT_ALIGN_LEFT, 
        x = display.right - 160,
        y = display.bottom + 120,
    })
    self.moneyLabel:setAnchorPoint(ccp(0,0.5))
    self.layer:addWidget(self.moneyLabel)

    self.timeLabel = ccs.labelAtlas({
        text = "00:00",
        start = '0',
        image = "ui/score/number_bar_time.png",
        w = 31,
        h = 33,
        align = ui.TEXT_ALIGN_LEFT, 
        x = display.left + 180,
        y = display.bottom + 120,
    })
    self.layer:addWidget(self.timeLabel)

    self.gotoNextButton:setEnabled(false)
    self.gotostoreButton:setEnabled(false)
    self.gotoNextButton:setVisible(false)
    self.gotostoreButton:setVisible(false)
end

function ScoreScene:updateUI()
 
    local data = GameSystem._record.lastGameInfo
    if data == nil then 
        CCLuaLog("[游戏数据丢失]")
        return
    end

    self.distanceBar:setItemInfo({
        desc = "路程:",
        num = data.distance,
        max = 5000,
        suffix = "m"
    })
    
    self.killBar:setItemInfo({
        desc = "杀敌数:",
        num = data.numberOfkillEnemy,
        max = 100,
        suffix = ""
    })
    
    self.comboBar:setItemInfo({
        desc = "连击数:",
        num = data.maxNumberOfCombs,
        max = 100,
        suffix = ""
    })

    self.speedBar:setItemInfo({
        desc = "平均速度:",
        num = math.floor(data.distance/data.startTime * 3.6),
        max = 80,
        suffix = ""
    })

    self.timeLabel:setStringValue(cf.timeConvert(data.startTime,"M:S"))

    local numberOfGlod = math.floor( data.numberOfkillEnemy * 10 + data.numberOfkillBoss * 100 + data.numberOfkillSuperBoss * 1000 + data.distance*0.5)
    numberOfGlod = numberOfGlod * LevelSystem._level.currentPlayMap
    data.numberOfGlod  = numberOfGlod
    EffectHelper.RollNumEffect(self.moneyLabel, 1.0 ,0, numberOfGlod  , "" ,"") 

    self.playerInfoLayer:setVisible(true)
 
    self:setupRecord(data)
    self.playerInfoLayer:updateTaskInfo(false)  
    audio.playEffect(R.SOUND_NUM_ROLL, false)    

    self:performWithDelay(function()
 
        self.pushLayer = BuyConfirmLayer.new()
        local itemId = self.pushLayer:getPushItem()
        if itemId then
            if not EquipSystem:checkIsExist(itemId) then
                 self.pushLayer:setItemIdToPush(itemId,"ui/level/ui_item_" .. itemId ..".png",function()
                 end)
                 self:addChild(self.pushLayer)
            end
        end

        self.gotoNextButton:setEnabled(true)
        self.gotostoreButton:setEnabled(true)
        self.gotoNextButton:setVisible(true)
        self.gotostoreButton:setVisible(true)
    end, 1.5)
 
end

--[[
    设置游戏记录
]]
function ScoreScene:setupRecord(data)

    GameSystem._record.distance = math.floor(GameSystem._record.distance + data.distance)
 
    GameSystem._record.numberOfkillEnemy = GameSystem._record.numberOfkillEnemy + data.numberOfkillEnemy
    GameSystem._record.numberOfkillBoss = GameSystem._record.numberOfkillBoss + data.numberOfkillBoss
    GameSystem._record.numberOfkillSuperBoss = GameSystem._record.numberOfkillSuperBoss + data.numberOfkillSuperBoss
    
    GameSystem._record.numberOfhitCars = GameSystem._record.numberOfhitCars + data.numberOfhitCars
    GameSystem._record.numberOfhitEnemy = GameSystem._record.numberOfhitEnemy + data.numberOfhitEnemy

    GameSystem._record.maxNumberOfCombs = math.max(GameSystem._record.maxNumberOfCombs , data.maxNumberOfCombs)
    GameSystem._record.maxNumberOfSpeed = math.max(GameSystem._record.maxNumberOfSpeed , data.maxNumberOfSpeed)

    GameSystem._record.numberOfFly = GameSystem._record.numberOfFly + data.numberOfFly
    GameSystem._record.numberOfHittings = GameSystem._record.numberOfHittings + data.maxNumberOfCombs
    GameSystem._record.numberOfGlod = GameSystem._record.numberOfGlod + data.numberOfGlod
    GameSystem._record.numberOfDeads = GameSystem._record.numberOfDeads + data.numberOfDeads

    GameSystem._record.maxTimeOfForeverMode = math.max(GameSystem._record.maxTimeOfForeverMode , data.maxTimeOfForeverMode)

    AchievementSystem:checkAll()
    GameSystem:save()

end

function ScoreScene:onEnter()
    
end

function ScoreScene:onEnterTransitionFinish()
    CCSpriteFrameCache:sharedSpriteFrameCache():removeUnusedSpriteFrames()
    CCTextureCache:sharedTextureCache():removeUnusedTextures()
end

function ScoreScene:onExit()
    
end

return ScoreScene