--*********************************************
-- Desc: 怪物游戏层
-- File: Monster.lua    
-- Author: zhouhaifeng
-- Date: 2014-03-12 00:23:16
--*********************************************

local Monster = import(".Monster")
local Boss = import(".Boss")
local Block = import(".Block")
local MapData = import("..datas.MapData")
local TaskInfoLayer =  import("..layers.TaskInfoLayer")
local ObjectsManager = {}

function ObjectsManager:reset()
	self.monstersList = {}
	self.blocksList = {}
	self.enemysList = {}
	self.bulletsList = {}
	self.bossList = {}
	self.effectList = {}

	self.numOfBoss = 0
	self.numOfEnemys = 0
	self.numOfBlocks = 0
	self.numOfMonsters = 0
	self.hasMadeAreaCount = -1
	self.noEnemyFlag = true

	self.tileWidth = 32
	self.tileHeight = 32
end

--[[
	随机生成地图
]]
function ObjectsManager:randomMakeMap(distance) 
	
	local currentArea = math.floor(distance/25) + 1
	if self.hasMadeAreaCount >= currentArea  then
		-- CCLuaLog("hasLoad:" .. currentArea .. "currentArea:" .. currentArea)
		return 
	end	

	local mapLayer = Global:get("mapLayer")
	local controlLayer = Global:get("controlLayer")


	if self.noEnemyFlag then
		return 
	end

	local map = LevelSystem:getCurrentPlayMap()
	local level = LevelSystem:getCurrentLevel()
	local config = nil

	local stepId = GameSystem:getGuideId()
    
    if stepId == kGUIDE_MOVE or stepId == kGUIDE_FIRE or stepId == kGUIDE_SPEED then

    	config = MapData.Templates[1]
   	
   	else
		if map == 1 then

			local count = math.fmod(self.hasMadeAreaCount,6)
			if count == 0 or count == 2 or count == 4 then
				config = MapData.Templates[1]
			elseif count == 1 or count == 3 then
				config = MapData.Templates[math.random(2,5)]
			else 
				config = MapData.Templates[math.random(6,15)]
			end

			if math.fmod(currentArea,10) == 0 then
				self:randomAddBoss("1012")
			end

		elseif map == 2 then 
			local count = math.fmod(self.hasMadeAreaCount,4)
			if count == 0 or count == 2  then
				config = MapData.Templates[1]
			elseif count == 1 then
				config = MapData.Templates[math.random(2,5)]
			else 
				config = MapData.Templates[math.random(11,20)]
			end

			if math.fmod(currentArea,25) == 0 then
				self:randomAddBoss("1012")
			end
			
		elseif map == 3 then
			local count = math.fmod(self.hasMadeAreaCount,4)
			if count == 0 or count == 2  then
				config = MapData.Templates[1]
			elseif count == 1 then
				config = MapData.Templates[math.random(2,5)]
			else 
				config = MapData.Templates[math.random(16,25)]
			end

			if math.fmod(currentArea,20) == 0 then
				self:randomAddBoss("1013")
			end

		elseif map == 4 then
			local count = math.fmod(self.hasMadeAreaCount,6)
			if count == 0 or count == 2  then
				config = MapData.Templates[1]
			elseif count == 1 then
				config = MapData.Templates[math.random(2,5)]
			elseif count == 3 or count == 5 then
				config = MapData.Templates[math.random(21,30)]
			end

			if math.fmod(currentArea,10) == 0 then
				self:randomAddBoss("1013")
			elseif math.fmod(currentArea,15) == 0 then
				self:randomAddBoss("1014")
			end
		elseif map == 5 then
			local count = math.fmod(self.hasMadeAreaCount,6)
			if count == 0 or count == 2  then
				config = MapData.Templates[1]
			elseif count == 1 then
				config = MapData.Templates[math.random(2,5)]
			elseif count == 3 or count == 5 then
				config = MapData.Templates[math.random(21,30)]
			end

			if math.fmod(currentArea,10) == 0 then
				self:randomAddBoss("1014")
			end
		end 
		
		--如果任务完成了
		controlLayer:checkTasks()
	end

	mapLayer:performWithDelay(function()

		if stepId == kGUIDE_FIRE then
			self:addMonsterForGuide()
			
		elseif stepId == kGUIDE_SPEED then
			self:addMonsterForGuide()
			self:addMonsterForGuide()
			self:addMonsterForGuide()

		elseif config then
			 self:randomAddMonster(config.monsters); 
			 mapLayer:performWithDelay(function()
		 		if config then
					self:randomAddBlock(config.blocks);
				end
			 end, 0)
		end

	 end, 0)

    self.hasMadeAreaCount = currentArea
end

--[[
	随机出怪
]]
function ObjectsManager:randomAddEnemy()

	if self.numOfEnemys >= 5 then
		return 
	end

	local controlLayer = Global:get("controlLayer")
	local player = Global:get("player")
	local mapLayer = Global:get("mapLayer")
	local MonsterPools= Global:get("MonsterPools")
	
	for i = 1 , 3 do
		local monster = MonsterPools:getAnObject("Monster_" .. math.random(1001,1006))
		monster:reset()

		local x = monster.params.pos.x  * self.tileWidth
		local y = (monster.params.pos.y + 0.3) * self.tileHeight + 64

		mapLayer:setObjectPosition(monster, x , y)

		--设置速度
		monster:doEvent("torun")
		monster:setMonsterSpeed(player.car.minSpeed,player.car.maxSpeed,controlLayer.record.distance)

		-- monster.avstar.nameLabel:setString(params.pos.y)
		self.enemysList[monster] = monster.params

		mapLayer.tileMap:addChild(monster)
	end
end

--[[
	随机出怪
]]
function ObjectsManager:randomAddBoss(type)

	self:showBossAlert()
	local controlLayer = Global:get("controlLayer")
	local player = Global:get("player")
	local mapLayer = Global:get("mapLayer")
	local MonsterPools= Global:get("MonsterPools")

	local monster = Boss.new(type)
	monster:doEvent("tostandby")
	
	local x = display.width + monster.params.pos.x  * self.tileWidth 
	local y = (monster.params.pos.y + 0.3)* self.tileHeight + 64

	mapLayer.tileMap:addChild(monster)
	mapLayer:setObjectPosition(monster, x , y)
	self.enemysList[monster] = monster.params	
end

--[[
	显示boss提示
]]
function ObjectsManager:showBossAlert()
	local mapLayer = Global:get("mapLayer")
	self.bossAlert = display.newSprite("ui/gameui/boss.png", display.cx, display.cy)
    display.setIsBackgroundImage(self.bossAlert)
    mapLayer:addChild(self.bossAlert,9999)
    audio.playEffect(R.SOUND_WARNING);
    self.bossAlert:runAction(CCSequence:createWithTwoActions(CCBlink:create(2, 2), CCRemoveSelf:create(true)))
end

--[[
	随机出怪
]]
function ObjectsManager:randomAddMonster(config)
	if config == nil then
		return 
	end

	if self.numOfMonsters >= 20 then
		return 
	end

	local mapLayer = Global:get("mapLayer")
	local MonsterPools = Global:get("MonsterPools")

	
	-- Add all Monsters
	for k,v in pairs(config) do
		local monster = MonsterPools:getAnObject("Monster_" .. mapLayer.monsters[math.random(1,#mapLayer.monsters)] )
		monster:reset()
		monster.params.pos = ccp(25+v.offsetX,v.offsetY)
		monster:doEvent("towalk")
		local x = monster.params.pos.x  * self.tileWidth 
		local y = (monster.params.pos.y +0.3)* self.tileHeight + 64

		mapLayer.tileMap:addChild(monster)
		
		mapLayer:setObjectPosition(monster, x , y)
		-- monster.avstar.nameLabel:setString(params.pos.y)
		self.monstersList[monster] = monster.params	
	end
end

--[[
	随机出怪
]]
function ObjectsManager:addMonsterForGuide()

	local mapLayer = Global:get("mapLayer")
	local MonsterPools = Global:get("MonsterPools")
	local player = Global:get("player")

	-- Add all Monsters
	local monster = MonsterPools:getAnObject("Monster_" .. mapLayer.monsters[math.random(1,#mapLayer.monsters)] )
	monster:reset()
	monster.params.pos = ccp(25,player.avstar.col)
	monster:doEvent("tostandby")
	local x = display.width + math.random(0,100)
	local y = player.avstar:getPositionY()

	mapLayer.tileMap:addChild(monster)
	
	mapLayer:setObjectPosition(monster, x , y)
	-- monster.avstar.nameLabel:setString(params.pos.y)
	self.monstersList[monster] = monster.params	
end



--[[
	随机添加障碍
]]
function ObjectsManager:randomAddBlock(config)
	if config == nil then
		return 
	end
	
	if self.numOfBlocks >=  10 then
		return 
	end

	local mapLayer = Global:get("mapLayer")
	local BlockPools = Global:get("BlockPools")

	for k,v in pairs(config) do

		local blocks = BlockPools:getAnObject()
		blocks:setBlockType(v.blockType)
		blocks.params.pos = ccp(25+v.offsetX,v.offsetY)
		blocks.params.enabled = true

		local x = blocks.params.pos.x * self.tileWidth 
		local y = blocks.params.pos.y * self.tileHeight + 64
		
		mapLayer.tileMap:addChild(blocks)
		mapLayer:setObjectPosition(blocks, x , y)

		-- blocks.nameLabel:setString(blocks:getZOrder())
		self.blocksList[blocks] = blocks.params
	end
end



--[[
	获取碰撞区域
]]
function ObjectsManager:getCollisionRect(object,params)
	return CCRectMake(
				object:getPositionX()+16,
				object:getPositionY()+16,
			    params.hitrect.width * self.tileWidth - 16,
			    params.hitrect.height * self.tileHeight - 16
			   )
end

--[[
	获取碰撞区域
]]
function ObjectsManager:getEventRect(object,params)
	return CCRectMake(
				object:getPositionX() + params.eventRect.origin.x,
				object:getPositionY() + params.eventRect.origin.y,
			    params.eventRect.size.width,
			    params.eventRect.size.height
			   )
end


--[[
	添加子弹的碰撞
]]
function ObjectsManager:addBulletCollision(bullet)
	if bullet then
		table.insert(self.bulletsList,bullet)
	end
end

--[[
	取消子弹的碰撞
]]
function ObjectsManager:removeBulletCollision(k)
	if k then
		self.bulletsList[k] = nil
	end
end

--[[
	添加特效的移动
]]
function ObjectsManager:addEffect(effect)
	if effect then
		table.insert(self.effectList,effect)
	end
end

--[[
	取消特效移动
]]
function ObjectsManager:removeEffect(k)
	if k then
		self.effectList[k] = nil
	end
end

--[[
	碰撞检测
]]
function ObjectsManager:collisionTest()

	local mapLayer = Global:get("mapLayer")
	local player = Global:get("player")
	local controlLayer = Global:get("controlLayer")
	local avstar = mapLayer.avstar;
	local rect = player:getCollisionRect()
	local eventRect = player:getTiggerRect()
	local stepId = GameSystem:getGuideId()

	--怪物碰撞
	if self.monstersList then
	    for monsterNode,params in pairs(self.monstersList) do
	    	if params.enabled then
				local contentRect = monsterNode:getCollisionRect()
		      	--那么执行子弹的碰撞...
	      		for index,bullet in pairs(self.bulletsList) do
	      			local bulletSize = bullet:getCollisionRect()
			      	local flag = bulletSize:intersectsRect(contentRect)
			      	if flag then
			      		-- CCLuaLog("hitted")
			   			monsterNode:doEvent("attacked")
			   			if monsterNode.config.isDead then
			   				params.enabled = false
			   				if stepId == kGUIDE_FIRE then
				   			  GameSystem:finishGuide(stepId)
				   			  controlLayer.guideLayer:removeFromParent()
	                          controlLayer:checkGuide() 
	                        end
			   			end
			   			bullet:onHitOne(index)
			      	end   
				end
		
				if params.enabled then
					--如果没有被撞死...
		      		local flag = rect:intersectsRect(contentRect)
					if avstar.col == params.pos.y and flag then
			      		params.enabled = false
			      		monsterNode:doEvent("todead","hited")
			      		player:doEvent("hited")

			      		if stepId == kGUIDE_SPEED then
                        	GameSystem:finishGuide(stepId)
                        	controlLayer.guideLayer:removeFromParent()
                        	controlLayer:checkGuide() 
                        	controlLayer:resetRecord()
			      		end
		      		end
				end

				--障碍
			    if params.enabled then 
				    for blockNode,blockParams in pairs(self.blocksList) do 
				    	if blockParams.enabled then
				    		local blockNodeRect = blockNode:getCollisionRect()
					       	local flag = contentRect:intersectsRect(blockNodeRect)
					       	if monsterNode.col == blockParams.pos.y and flag then
					       		if monsterNode.config.isBoss then
					       			blockNode:onExplode()
						      		params.enabled = false
						      		player:doEvent("hitedcar")
						      	end

						      	if params.enabled and flag then
					       			--碰到障碍物以后动作停止
					       			monsterNode.config.isStuck = true
					       			monsterNode:doEvent("tostandby")
					      		end
					      	end  	

					      	
					    end
				    end
				end
 	
	    	end
	    end
	end

	  -- 怪物
	 if self.enemysList then
	    for enemyNode,params in pairs(self.enemysList) do
	    	if params.enabled then
	    		local contentRect = enemyNode:getCollisionRect()
		    	--那么执行子弹的碰撞...
	      		for index,bullet in pairs(self.bulletsList) do
	      			local bulletSize = bullet:getCollisionRect()
			      	local flag = bulletSize:intersectsRect(contentRect)
			      	if flag then
			      		-- CCLuaLog("打死了一只小怪...")
			   			enemyNode:doEvent("attacked")
			   			if enemyNode.config.isDead then
			   				params.enabled = false
			   			end
			   			bullet:onHitOne(index)
			      	end
				end

				if params.enabled then
					if  enemyNode.col == avstar.col and rect:intersectsRect(contentRect) then
						enemyNode:doEvent("todead")
			      		if enemyNode.config.isDead then
			   				params.enabled = false
						end
			      		player:doEvent("hitedcar")
			      	end
				end

				--障碍
			    if params.enabled then 
				    for blockNode,blockParams in pairs(self.blocksList) do 
				    	if blockParams.enabled then
				    		local blockNodeRect = blockNode:getCollisionRect()
					       	local flag = contentRect:intersectsRect(blockNodeRect)
					       	if enemyNode.col == blockParams.pos.y and flag then
					       		if enemyNode.config.isBoss then
					       			blockNode:onExplode()
						      		params.enabled = false
					       		end
					      	end  	
					    end
				    end
				end
			end
	    end
	 end

    --障碍
    if self.blocksList then 
	    for blockNode,params in pairs(self.blocksList) do 
	    	if params.enabled then
		       	local flag = eventRect:intersectsRect(self:getEventRect(blockNode,params))
		       	if avstar.col == params.pos.y and flag then
					--无视障碍状态
					if player.config.collisionEnabled then 
						if params.blockType == 1 then
			       			-- CCLuaLog("我fei了....")
				      		params.enabled = false
				      		player:doEvent("tofly")
			       		else 
			       			-- CCLuaLog("我撞车了....")
			       			blockNode:onExplode()
				      		params.enabled = false
				      		player:doEvent("hitedcar")
			       		end
					else
						if player.config.isFly then
							return 
						end
						 -- 摧毁汽车
						if params.blockType == 1 then
			       			-- CCLuaLog("无视碰撞。。。坡")
			       			params.enabled = false
				      		player:doEvent("tofly")
			       		else 
			       			blockNode:onExplode()
			       			params.enabled = false
			       			 -- CCLuaLog("摧毁汽车....")
			       		end
					end
		      	end  

		      	if params.enabled and params.blockType == 2 then
			      	local contentRect = blockNode:getCollisionRect()
			      	--那么执行子弹的碰撞...
		      		for index,bullet in pairs(self.bulletsList) do
		      			local bulletSize = bullet:getCollisionRect()
				      	local flag = bulletSize:intersectsRect(contentRect)
				      	if flag then
				   			blockNode:onHit(player:getAttackDamage())
				   			if blockNode.params.isDestory then
				   				blockNode:onExplode()
				   				params.enabled = false
				   			end
			       			
				   			bullet:onHitOne(index)
				      	end   
					end	
				end

		    end
	    end
	end
end

--[[
	杀死所有的怪物
]]
function ObjectsManager:killAllEnemys()
    if self.enemysList then 
	    for monsterNode,params in pairs(self.enemysList) do 
	    	if params.enabled then
      			monsterNode:doEvent("todead")
      			params.enabled = false
		    end
	    end
	end
end   

--[[
	炸弹爆炸僵尸
]]
function ObjectsManager:killWithBomb(collisionRect)
	
	if self.enemysList then
		for enemyNode,params in pairs(self.enemysList) do
  			if params.enabled then
	  			local contentSize = enemyNode:getCollisionRect()
	  		 	local flag = collisionRect:intersectsRect(contentSize)
		      	if flag then
		      		if not enemyNode.config.isBoss then
		      			enemyNode.config.isBurn = true
		      		end
		   			enemyNode:doEvent("attacked")
		   			if enemyNode.config.isDead then
		   				params.enabled = false
		   				if not enemyNode.config.isBoss then
		   					params.isBurn = true
		   				end
		   			end
		      	end
			end
		end
	end


	if self.monstersList then
		for monsterNode,params in pairs(self.monstersList) do
  			if params.enabled then
	  			local contentSize = monsterNode:getCollisionRect()
	  			local flag = collisionRect:intersectsRect(contentSize)
		      	if flag then
		   			monsterNode:doEvent("attacked")
		   			if monsterNode.config.isDead then
		   				params.enabled = false
		   			end
		      	end
			end
		end
	end

	 if self.blocksList then 
		for blockNode,params in pairs(self.blocksList) do
  			if params.enabled and params.blockType == 2 then
	  			local contentSize = self:getEventRect(blockNode,params)
	  			local flag = collisionRect:intersectsRect(contentSize)
		      	if flag then
		   			blockNode:onExplode()
		   			params.enabled = false
		      	end
			end
		end
	end
end



--[[
	待机所有的怪物
]]
function ObjectsManager:idleAllEnemeys()
    if self.enemysList then 
	    for monsterNode,params in pairs(self.enemysList) do 
	    	if params.enabled then
      			monsterNode:doEvent("tostandby")
      			monsterNode:setScale(1)
		    end
	    end
	end
end      


--[[
	判断是否可以移动主角
]]
function ObjectsManager:canPlayerMoveTo(x,y)
	local player = Global:get("player")
	
	if player == nil then
		return false
	end

	local avstar = player.avstar;
	local rect = player:getTiggerRect()
	 --障碍
    if self.blocksList then 
	    for blockNode,params in pairs(self.blocksList) do 
	    	if params.enabled then
		       	local flag = rect:intersectsRect(blockNode:getCollisionRect())
		       	if flag and player.avstar.col == params.pos.y then
		       		if params.blockType == 1 then
		       			
		       		else
		       			blockNode:onExplode()
		       			params.enabled = false
		       			player:doEvent("hitedcar")
		       			return false
		       		end
		      	end  	
		     end
	     end
	end
	return true
end



--[[
	移动
]]
function ObjectsManager:movmentAction(dt,moveDistance)
	self.numOfBoss = 0
	self.numOfEnemys = 0
	self.numOfBlocks = 0
	self.numOfMonsters = 0
	 --怪物
	 local count = 0
	 local player = Global:get("player")
	 if self.monstersList then
	 	local MonsterPools= Global:get("MonsterPools")
	    for monsterNode,params in pairs(self.monstersList) do
	        monsterNode:setPositionX(monsterNode:getPositionX() - moveDistance + monsterNode.config.currentSpeed * dt);  
	        if monsterNode:getPositionX() + monsterNode:getContentSize().width <= -100 then   
	        	self.monstersList[monsterNode] = nil
	        	monsterNode:removeFromParentAndCleanup(false)
	        	MonsterPools:releaseObjects(monsterNode,"Monster_" .. monsterNode.monsterType)
	       	else
	       		self.numOfMonsters = self.numOfMonsters + 1
	       	end
	    end
	 end

	  -- 怪物
	 if self.enemysList then
	    for enemyNode,params in pairs(self.enemysList) do
	    	if params.enabled or params.isBurn then
	    		 enemyNode:updateLogic(player)	
	    	end

	        enemyNode:setPositionX(enemyNode:getPositionX() - moveDistance) 
	       
	       if enemyNode:getPositionX() + enemyNode:getContentSize().width <= -100 then   
	        	self.enemysList[enemyNode] = nil
	        	enemyNode:removeFromParentAndCleanup(true)
	        else
	        	self.numOfEnemys = self.numOfEnemys + 1
	        end
	    end
	 end

    --障碍
    if self.blocksList then 
    	local BlockPools= Global:get("BlockPools")
	    for blockNode,params in pairs(self.blocksList) do 
	        blockNode:setPositionX(blockNode:getPositionX() - moveDistance);  
	        if blockNode:getPositionX() + blockNode:getContentSize().width <= 0 then   	      
	        	blockNode:removeFromParentAndCleanup(false)
	           	BlockPools:releaseObjects(blockNode)
          		self.blocksList[blockNode] = nil
	        else
	        	self.numOfBlocks = self.numOfBlocks + 1
	        end
	    end
	end

	--移除粒子效果...
	for index,effect in pairs(self.effectList) do
		if effect:getPositionX() + effect:getContentSize().width <= -100 then
			effect:removeFromParentAndCleanup(true)
			self:removeEffect(index)
		else
			effect:setPositionX(effect:getPositionX() - moveDistance);  
		end
	end

	--移除子弹的碰撞...
	for index,bullet in pairs(self.bulletsList) do
		if not bullet.enabled then
			self:removeBulletCollision(index)
		end
	end
end


--[[
	更新移动距离 和 碰撞检测
]]
function ObjectsManager:updateFrame(dt,moveDistance)
	--移动
	self:movmentAction(dt, moveDistance)
	-- 碰撞检测
	self:collisionTest()
	
end


return ObjectsManager