--***********************************************
-- Desc: 主角
-- File: Player.lua    
-- Author: zhouhaifeng
-- Date: 2014-03-09 13:32:28
--***********************************************

local EffectManager = import("app.sprites.EffectManager")
local EquipData = import("app.datas.GameItems")
local Player = class("Player")
--[[
	初始化
	params: 角色信息
]]
function Player:ctor(params)
    self:loadConfig(params);
    self:loadActions();
	self:initFSM();

    self.avstar:setAnchorPoint(ccp(0.5,0.5))
   
    -- local width = self.avstar:getContentSize().width
    -- local height = self.avstar:getContentSize().height
    -- local rect = display.newRect(width,height)
    -- rect:setAnchorPoint(ccp(0,0))
    -- rect:setPosition(0,height/2)
    -- self.avstar:addChild(rect)

    -- local rect1 = display.newRect(self:getCollisionRect())
    -- rect1:setPosition(0,0)
    -- self.avstar:addChild(rect1)

    --  local rect1 = display.newRect(self:getTiggerRect())
    -- self.avstar:addChild(rect1)

end


function Player:reset()

    -- 获取装备的枪支信息
    local gunItemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
    self.gun = EquipSystem:getEquipProp(gunItemId) or {}
    self.gun.currentBullets = self.gun.bullet
    self.gun.coolTime = 0.5 * self.gun.shootSpeed
    self.gun.isCooling = false
    self.gun.isReloading = false
    self.gun.reloadProgress = 0
    self.gun.lastFireTime = os.clock()
    self:getFireAnimationName()
    --获取装备的摩托车的属性
    local carItemId = EquipSystem:getEquipItemWithType(kItemTypeCars)
    self.car = EquipSystem:getEquipProp(carItemId) or {}
    --获取装备的道具信息
    local propItemId = EquipSystem:getEquipItemWithType(kItemTypeProps)
    self.prop = EquipSystem:getEquipProp(propItemId) or {}
    --获取装备的服装信息
    local clothesItemId = EquipSystem:getEquipItemWithType(kItemTypeClothes)
    self.clothes = EquipSystem:getEquipProp(clothesItemId) or {}

    --初始化玩家属性
    self.config.sensitivity = 6                    -- 横向速度
    self.config.velocity = (self.car.maxSpeed - self.car.minSpeed) / 80.0 -- 加速度
  
    self.config.currentSpeed = self.car.minSpeed    -- 当前速度
    self.config.currentCombo = 0                    -- 当前连击数
    self.config.lastComboTime =  os.time()          -- 上次连击时间
    self.config.maxHp = self:calculateHP()
    self.config.hp = self.config.maxHp
    self.config.col = 0
    self.config.isDead = false
    self.config.isFly = false
    self.config.fireHold = false
    self.config.currentActionName = ""
    self.config.hitrect = self.car.hitrect
    self.config.eventRect = CCRectMake(0, 10, 80, 20)    --事件触发区域
    self.config.collisionEnabled = true
    self.config.isBulletInfinite = false                 --子弹无限
    self.config.hasCrashBarrier = false                  --防撞
    self.config.isNo2SpeedUp = false                     --氮气加速
    self.config.isSpeedUp = false

    --判断道具是否装备
    if tostring(carItemId) == "1006" then
       self.config.hasCrashBarrier = true
    end

end

--[[
    计算玩家最大血量
]]
function Player:calculateHP()
    return self.car.hp + self.clothes.hp
end

function Player:loadConfig(params)
    self.config = {}
    -- 单场的记录
    self:reset()
end

--[[
	初始化状态机
]]
function Player:initFSM()

	self.fsm_ = {}
    cc.GameObject.extend(self.fsm_)
        :addComponent("components.behavior.StateMachine")
        :exportMethods()

    self.fsm_:setupState({
        events = {
            --停止
            {name = "stop",  to = "stop" },
        	--开始
            {name = "startup", from = {"start","none","stop", "fire","fly"} , to = "start"},
            --复活
            {name = "reborn",  to = "start"},
            --加速
            {name = "speedup", from = "stop",  to = "start" },
            {name = "speedup",  from = {"speed","start","fire"},  to = "speed"},
            --减速
            {name = "speeddown",  from = {"start","speed","fire"},  to = "start"},
            --射击
            {name = "shoot",  from = {"none","start","stop","speed","fly","fire"},  to = "fire"},
            --受到攻击
            {name = "attacked"},
            --撞击怪物
            {name = "hited"},
            --撞击怪物CAR
            {name = "hitedcar"},
            --跳跃飞行
            {name = "tofly",  {"none","start","stop", "fire"}, to="fly"},
            --死亡
            {name = "todead", from = {"none","start","stop", "fire","fly"}, to="dead"},
        },
        callbacks = {
            onenterstart  = function(event)
                            if not self.config.isNo2SpeedUp then
                                 self:showMotoEffect()
                            end
                            self:playAction("start")
                            if self.config.currentSpeed > 0 then
                                 if self.config.currentSpeed <= 0 then
                                     self:doEvent("stop")
                                 end
                            end
                        end,
            onenterspeed   = function(event)
                            audio.playEffect(R.SOUND_MOTO_SPEED, false)

                            if not self.config.isNo2SpeedUp then
                                self.sootEffect = EffectManager:playSootEffect(self.avstar)
                                self:hideMotoEffect()
                            end
                           
                            -- CCLuaLog("[FSM] onbeforespeed with event " .. event.name) 
                            -- self:speedUpAnimation()
                            self:playAction("speedup",function(state)   
                                if state == 1 or state == 2 then
                                     event.transition()
                                end
                            end)  
                            return "async"
                        end,
            onspeedup     = function (event)
                            if self.config.currentSpeed < self.car.maxSpeed then
                                self.config.currentSpeed = self.config.currentSpeed + self.config.velocity 
                            end
                             self:playAction("speed")  
                        end,

            onleavespeed = function(event) 
                                if not self.config.isNo2SpeedUp then
                                    if self.sootEffect then
                                        self.sootEffect:removeFromParentAndCleanup(true)
                                        self.sootEffect = nil
                                    end
                                end 
                                -- CCLuaLog("[FSM] onleavespeed with event " .. event.name) 
                                if self.config.currentActionName == "speed" then
                                    self:playAction("speeddown",function(state)   
                                        if state == 1 or state == 2 then
                                           event.transition()
                                        end
                                    end)   
                                    return "async"
                                end                               
                        end,
            onhited = function(event) 
                            if not self.config.isNo2SpeedUp then 
                                 if self.config.currentSpeed >= self.car.minSpeed then
                                    local speeddown = self.config.velocity * 10
                                    if self.config.isSpeedUp then
                                        speeddown = speeddown / 2
                                    end
                                    self.config.currentSpeed = self.config.currentSpeed - speeddown
                                 end

                                 if self.config.currentSpeed < self.car.minSpeed then
                                    self.config.currentSpeed = self.car.minSpeed 
                                 end

                                 if self.config.currentSpeed <= 0 then
                                     self:doEvent("stop")
                                 end     
                            end
                        end,
            onhitedcar = function(event) 
                            if not self.config.isNo2SpeedUp then
                                if self.config.currentSpeed >= self.car.minSpeed then
                                    local speeddown = self.config.velocity * 20
                                    if self.config.isSpeedUp then
                                        speeddown = speeddown / 2
                                    end
                                    self.config.currentSpeed = self.config.currentSpeed - speeddown
                                end
                            end

                             if self.config.currentSpeed < self.car.minSpeed then
                                self.config.currentSpeed = self.car.minSpeed 
                             end
                            
                            if not self.config.hasCrashBarrier then
                                local damage = 4
                                if self.config.isSpeedUp then
                                    damage = damage / 2
                                end
                                self.config.hp = self.config.hp - damage
                            end

                            if self.config.hp < 0 then
                               self.config.hp = 0
                            end

                             if self.config.hp > 0 then
                                DataCenter:dispatchEvent({name = "PLAYER_ATTACKED"})    
                                if self.config.currentSpeed <= 0 then
                                    self:doEvent("stop")
                                end   
                             else
                                self:doEvent("todead")
                             end

                        end,
            onspeeddown     = function (event)
                            if self.config.isDead or self.config.isFly then
                                 return false
                            end

                            if  self.config.currentSpeed > self.car.minSpeed then
                                 self.config.currentSpeed = self.config.currentSpeed - self.config.velocity 
                            end
                             -- CCLuaLog("[FSM] onspeeddown" .. self.config.currentSpeed) 
                            if self.config.currentSpeed <= 0 then
                                 self:doEvent("stop")
                            end
                        end,
            onshoot = function (event)
                                -- print("self.gun.currentBullets:" .. self.gun.currentBullets)
                                --判断还有没有子弹
                                if not self:canFire() then
                                    self:doEvent("startup") 
                                    return false
                                end
                                --如果不是子弹无限的话
                                if not self.config.isBulletInfinite then
                                    if self.gun.weaponType == kWeaponTypeShotGun then
                                        self.gun.currentBullets = self.gun.currentBullets - 2
                                    else
                                        self.gun.currentBullets = self.gun.currentBullets - 1
                                    end    
                                end
                                
                                self:playAction(self.gun.fireAnim,function(state)  
                                    if state == 0 then
                                        self.gun.lastFireTime = os.clock()
                                        self.gun.isCooling = true
                                        if self.gun.weaponType == kWeaponTypeRPG then
                                             self:playBulletAnimtion(1)    
                                        end 
                                                                
                                    elseif state == 1 then
                                        if self.gun.weaponType ~= kWeaponTypeRPG then
                                          if self.gun.weaponType == kWeaponTypeShotGun then
                                            self:playBulletAnimtion(2)
                                          else
                                            self:playBulletAnimtion(1)
                                          end
                                        end
                                        self.gun.isCooling = false 
                                        if self.config.fireHold then
                                            self:doEvent("shoot")
                                        else
                                            self:doEvent("startup") 
                                        end
                                    end          
                                end)  

                                if self.gun.currentBullets == 0 then                               
                                    self:reloadBullets()   
                                end  
                            end,
            onstop = function(event) 
                            self:hideMotoEffect()
                           --CCLuaLog("[FSM] onstop") 
                            self.config.currentSpeed = 0
                            self:playAction("stop")  
                        end,
            ontodead = function(event)
                            self:hideMotoEffect()
                            self.config.isDead = true
                            self.config.currentSpeed = 0
                            self.config.hp = 0
                            DataCenter:dispatchEvent({name = "PLAYER_DEAD"})  
                            self:playAction("dead")    
                            audio.playEffect(R.SOUND_PLAYER_DEAD)
                        end,
            onattacked = function(event)
                            if self.config.isDead == true then
                                return
                            end

                            local damage = event.args[1] or 0
                            self.config.hp = self.config.hp - damage 
                            

                            if self.config.hp <= 0 then
                               self.config.hp = 0
                            end

                            local action1 = CCTintTo:create(0.1,180, 0, 0)
                            local action2 = CCTintTo:create(0.1,255, 255, 255)
                            self.avstar:runAction(CCSequence:createWithTwoActions(action1,action2))
                            
                            DataCenter:dispatchEvent({name = "PLAYER_ATTACKED"})      
                              --收到攻击
                            if self.config.hp == 0 then

                               self:doEvent("todead")
                            end                         
                        end,
            onreborn = function(event)
                            self:reset()
                            self.config.collisionEnabled = false
                            self.config.currentSpeed = 20
                            local blink = CCBlink:create(1.5, 3)
                            local callback = CCCallFunc:create(function ()
                                DataCenter:dispatchEvent({name = "GAME_START"})    
                                self.config.collisionEnabled = true  
                                self:doEvent("startup")   
                            end)
                            local action = CCSequence:createWithTwoActions(blink, callback)
                            self.avstar:runAction(action)

                            self:showMotoEffect()
                        end,
             onbeforetofly = function(event)
                                self:hideMotoEffect()
                             -- CCLuaLog("[FSM] LEAVE STATE: onbeforetofly")
                                if self.config.currentSpeed < 40 then
                                     self.config.currentSpeed = 40   
                                end

                                self.config.collisionEnabled = false
                                self.config.isFly = true
                                DataCenter:dispatchEvent({name = "PLAYER_FLY"})    
                                self:playAction("flyup",function (state)
                                    if state == 1 or state == 2 then
                                        -- CCLuaLog("flyup=====>"..state)
                                        
                                        self.config.isFly = false  
                                        self:playAction("flydown",function(flystate)
                                            if flystate == 1 or flystate == 2 then   
                                                -- CCLuaLog("flydown=============>"..flystate)  
                                                self.gun.isCooling = false 
                                                self.config.isFly = false
                                                self.config.collisionEnabled = true  
                                                self:doEvent("startup")

                                                self:playShockwaveEffect()
                                            end
                                       end) 
                                    end
                                end)
                            end,
        },
    })
end

--[[
	加载动画效果
]]
function Player:loadActions(params)

    self.animationNames = {
        start = "start",
        speedup = "speedup",
        speed = "speed",
        speeddown = "speeddown",
        stop = "stop",
        dead = "dead",
        firefast = "firefast",
        firenormal = "firenormal",
        fireslow = "fireslow",
        flyup = "flyup",
        fly = "fly",
        flydown = "flydown",
    }

	local manager = CCArmatureDataManager:sharedArmatureDataManager()
    local animationFileName = "animations/" .. self:getAnimationFileName() .. "/" .. self:getAnimationFileName()
    manager:addArmatureFileInfo(animationFileName..".ExportJson")

    self:updateClothes()
    
    self:addMotoEffect()
end

--[[
    获取动画名称
]]
function Player:playNo2SpeedUp()
    if self.speedupEffect then 
        self.speedupEffect:removeFromParentAndCleanup(true)
        self.speedupEffect = nil
    end 

    if self.fireEffect then 
        self.fireEffect:removeFromParentAndCleanup(true)
        self.fireEffect = nil
    end
        
    if self.sootEffect then
        self.sootEffect:removeFromParentAndCleanup(true)                             
        self.sootEffect = nil
    end

    self.speedupEffect = CCArmature:create("speedup")
    self.speedupEffect:setScale(0.6)
    self.speedupEffect:setPosition(self.car.no2Pos)
    self.speedupEffect:getAnimation():play("effects_chong")
    self.avstar:addChild(self.speedupEffect,999)

    self.fireEffect = CCArmature:create("shockwave")
    self.fireEffect:setScale(0.6)
    self.fireEffect:setPosition(ccp(-95,18))
    self.fireEffect:getAnimation():play("fire")
    self.avstar:addChild(self.fireEffect,999)

    self:hideMotoEffect()
end

--[[
    冲击波特效
]]
function Player:playShockwaveEffect()
    if self.flydownEffect then 
    self.flydownEffect:removeFromParentAndCleanup(true)
    end 

    self.flydownEffect = CCArmature:create("shockwave")
    self.flydownEffect:setPosition(ccp(0,20))
    self.flydownEffect:getAnimation():play("shockwave")
    self.avstar:addChild(self.flydownEffect)

    self.flydownEffect:runAction(CCSequence:createWithTwoActions(CCDelayTime:create(0.6), CCCallFunc:create(function()
        if self.flydownEffect then 
            self.flydownEffect:removeFromParentAndCleanup(true)
            self.flydownEffect = nil
        end 
    end)))

    local objectsManager = Global:get("objectsManager")
    objectsManager:killWithBomb(self.flydownEffect:getBoundingBox())
end

--[[
    获取动画名称
]]
function Player:getAnimationFileName()
   return "moto_animation_" .. (self.car.itemId - 1000)
end

--[[
    发射子弹
]]
function Player:playBulletAnimtion(num)
    --添加子弹效果
    
    local BulletPools = Global:get("BulletPools")
    local mapLayer = Global:get("mapLayer")

    local bullet = BulletPools:getAnObject()
    bullet:setBulletType(self.gun.weaponType)
    local x,y = self.avstar:getPosition()
    pos = mapLayer:convertToNodeSpace(ccp(x,y))
    pos = ccpAdd(pos,self.car.firepos)
    pos = ccpAdd(pos,self.gun.firepos)
    bullet:setPosition(pos.x,pos.y)

    bullet:onFire()

    if num == 2 then
        local bullet = BulletPools:getAnObject()
        bullet:setBulletType(self.gun.weaponType)
        pos = ccpAdd(pos, ccp(0,10))
        bullet:setPosition(pos.x,pos.y)
        bullet:onFire()
    end
   

end

--[[
    执行事件
]]
function Player:doEvent(eventName , ...)
    --设置不可操作操作状态
    if self.config.isFly then
        return
    end
   
    if eventName == "attacked" or eventName == "hited" or eventName == "hitedcar" then
         self.fsm_:doEventOnly(eventName, ...)
    elseif self.fsm_:canDoEvent(eventName) and not self.config.isDead then 
         self.fsm_:doEvent(eventName , ...)
    elseif (eventName == "tofly" or eventName == "reborn" or eventName == "todead") then
         self.fsm_:doEventForce(eventName , ...)
    end
end

--[[
    是否是某个状态
]]
function Player:isState(state)
    return self.fsm_:isState(state)
end

--[[
    是否可以发射子弹
]]
function Player:canFire()
     if self.gun.currentBullets > 0 then
        local now = os.clock()
        if now - self.gun.lastFireTime >= self.gun.coolTime then
            self.gun.isCooling = false                                                                          
        end       

        if self.config.isDead then
            return false
        elseif self.gun.isReloading then
            return false
        elseif self.gun.isCooling then
            return false
        else
            return true
        end
    else
        self:reloadBullets()
    end
    return false
end

--[[
    换弹夹
]]
function Player:reloadBullets()
    if not self.gun.isReloading then
         self.gun.isReloading = true
         self.gun.currentBullets = 0
         scheduler.performWithDelayGlobal(function()
             self.gun.currentBullets = self.gun.bullet
             self.gun.isReloading = false
         end, self.gun.fillSpeed)
    end
end

--[[
    攻击速度
]]
function Player:getFireAnimationName()
    if self.gun.shootSpeed == 1 then 
         self.gun.fireAnim = "firefast"
    elseif self.gun.shootSpeed == 2 then 
        self.gun.fireAnim = "firefast"
    elseif self.gun.shootSpeed == 3 then
        self.gun.fireAnim = "fireslow"
    end
    
end

--[[
    计算攻击伤害
]]
function Player:getAttackDamage()
    return math.floor(math.random(self.gun.minHurt,self.gun.maxHurt) * self.clothes.hurt)
end

--[[
    是否是ComBo
]]
function Player:isCombo()
    if self.config.lastComboTime then
        local timeoffset =  os.time() - self.config.lastComboTime

        self.config.lastComboTime = os.time()
         if timeoffset > 5 then
             -- cleanup combo
             self.config.currentCombo = 1
             return false
        else
            -- show combo
            self.config.currentCombo =  self.config.currentCombo + 1
            return true
        end

        return true;
    else
        --first time
        self.config.currentCombo = 1
        self.config.lastComboTime = os.time()
        return false;
    end
end

--[[
    添加摩托车特效，尾气和泥土
]]
function Player:addMotoEffect()
    self.exhaustEffect = EffectManager:playExhaustEffect(self.avstar)
    self.exhaustEffect:setPosition(ccp(-95,18))
end

--[[
    播放摩托车特效
]]
function Player:showMotoEffect()
    self.exhaustEffect:setVisible(true)
end

--[[
    隐藏摩托车特效
]]
function Player:hideMotoEffect()
    self.exhaustEffect:setVisible(false)
end


--[[
    上下移动动画
]]
function Player:moveAction(eventName)
    if self.config.currentSpeed == 0 then
        return 
    end

    local mapLayer = Global:get("mapLayer")
    local x,y = self.avstar:getPosition()
    
    if eventName == "moveup" then
        y = y + self.config.sensitivity
        if y > 304 then
            y = 304
        end
        mapLayer:setPlayerPosition(eventName,x,y)
    elseif eventName == "movedown" then
        if y < 64 then
            y = 64
        end
        y = y - self.config.sensitivity
        mapLayer:setPlayerPosition(eventName,x,y)
    end
end

--[[
	播放动画
	params: 动画名称
]]
function Player:playAction(actionName,complete)
    local aniName = self.animationNames[actionName]
    if aniName == nil then
        CCLuaLog("[Lua Error] inivailed actionName:" .. actionName)
        return 
    end

    if actionName == "speedup" or actionName == "speeddown" or actionName == "start" then
        self.animation:setSpeedScale(0.8+ self.config.currentSpeed/100)
    elseif actionName == "firefast" or actionName == "fireslow" or actionName == "firenormal" then
        self.animation:setSpeedScale(1.2);
    elseif actionName == "flyup" or actionName == "flydown" or actionName == "fly" then
        self.animation:setSpeedScale(2.0)
    else
        self.animation:setSpeedScale(1)
    end
    
    if complete then
        self.movements[aniName] = complete
    end
 
     -- CCLuaLog("[Lua] play actionName:" .. actionName .. " aniName:" ..aniName)
    -- if self.config.currentActionName ~= actionName then    
        
         self.animation:play(aniName)
         self.config.currentActionName = actionName
    -- end
end

--[[
    碰撞检测的范围
]]
function Player:getCollisionRect()
    return CCRectMake(
                        self.avstar:getPositionX() + self.config.hitrect.origin.x,
                        self.avstar:getPositionY() + self.config.hitrect.origin.y,
                        self.config.hitrect.size.width - 1,
                        self.config.hitrect.size.height - 1
              )
end

--[[
    事件触发的范围
]]
function Player:getTiggerRect()
    return CCRectMake(
                        self.avstar:getPositionX() + self.config.eventRect.origin.x,
                        self.avstar:getPositionY() + self.config.eventRect.origin.y,
                        self.config.eventRect.size.width,
                        self.config.eventRect.size.height
                     )
end


--[[
	换装系统
	params:道具信息
]]
function Player:updateClothes()
	--保存
	local stateParams = self:saveStateforUpdateClothes()
	--读取配置
    -- self:checkPropForUpdateCars()
	self:checkPropForUpdateClothes();
    self:checkPropForUpdateWeapons();
    --获取新的状态
    if self.avstar == nil then
        self.avstar = CCNodeExtend.extend(CCArmature:create(self:getAnimationFileName()))
    end
    
    if self.animation then
        self.animation:removeScriptMovementEventHandler()
    end

    self.animation = self.avstar:getAnimation()
    self.movements = {}
    self.animation:setScriptMovementEventHandler(function(movementId, state)
        local complete = self.movements[movementId]
        if complete then
            complete(state)
        end
    end)
    self.animation:setSpeedScale(1)
    -- 还原
   	self:resumeStateAfterUpdateClothes(stateParams)
   	stateParams = nil
end

--===============================================================
--   						私有方法
--===============================================================

--[[
	更新纹理
	纹理名称，替换纹理
]]
function Player:updateSpriteFrame(frameName, frame)
	local spriteCache = CCSpriteFrameCache:sharedSpriteFrameCache();
    spriteCache:removeSpriteFrameByName(frameName);
    spriteCache:addSpriteFrame(frame,frameName);
end

--[[
    更新纹理
    纹理名称，替换纹理
]]
function Player:updateSpriteFrameFile(plistFile, newPlistFile ,newPngImage)
    local spriteCache = CCSpriteFrameCache:sharedSpriteFrameCache();
    spriteCache:removeSpriteFramesFromFile(plistFile);
    display.addSpriteFramesWithFile(newPlistFile, newPngImage)
end

--[[
    开始换装
]]
function Player:startChanging(params)
    if type(params) == "table"  then
        local frameName = nil
        local frame = nil
        for k,v in pairs(params) do
            local ext = v.file:match(".+%.(%w+)$")
            ext = string.lower(tostring(ext))
            if ext == "plist" then 
                self:updateSpriteFrameFile(v.file,v.file,v.image)
            else
                frameName = v.file
                frame = CCSpriteFrame:create(v.image,v.rect)
                self:updateSpriteFrame(frameName,frame)   
            end
        end
    end
end


--[[
	保存状态
]]
function Player:saveStateforUpdateClothes()
	local params = nil

	if self.avstar then
		params = {}
		params.parent = self.avstar:getParent()
		params.pos = ccp(self.avstar:getPositionX(),self.avstar:getPositionY())
		params.zorder = self.avstar:getZOrder()
		
		self.avstar:removeFromParentAndCleanup(true)
		self.avstar = nil
	end

	return params
end

--[[
	从配置文件中获取装备，道具属性
]]
function Player:checkPropForUpdateClothes()
	local data = {
       ['3001'] = {  }, 
       ['3002'] = { {file = "animations/body_1.plist" , image = "animations/body_1.png"} },
       ['3003'] = { {file = "animations/body_2.plist" , image = "animations/body_2.png"} },
       ['3004'] = { {file = "animations/body_3.plist" , image = "animations/body_3.png"} }    
    }

    local params = data[tostring(self.clothes.itemId)]
    self:startChanging(params)
end

--[[
    从配置文件中获取装备，道具属性
]]
function Player:checkPropForUpdateWeapons()
    local data = {
       ['2000'] = {  }, 
       ['2001'] = { {file = "animations/guns_1.plist" , image = "animations/guns_1.png"} },
       ['2002'] = { {file = "animations/guns_2.plist" , image = "animations/guns_2.png"} },
       ['2003'] = { {file = "animations/guns_3.plist" , image = "animations/guns_3.png"} },
       ['2004'] = { {file = "animations/guns_4.plist" , image = "animations/guns_4.png"} },
       ['2005'] = { {file = "animations/guns_5.plist" , image = "animations/guns_5.png"} },
       ['2006'] = { {file = "animations/guns_6.plist" , image = "animations/guns_6.png"} },    
       ['2007'] = { {file = "animations/guns_7.plist" , image = "animations/guns_7.png"} },
       ['2008'] = { {file = "animations/guns_8.plist" , image = "animations/guns_8.png"} },
       ['2009'] = { {file = "animations/guns_9.plist" , image = "animations/guns_9.png"} },    
    }

    local params = data[tostring(self.gun.itemId)]
    self:startChanging(params)
end


--[[
	还原状态
]]
function Player:resumeStateAfterUpdateClothes(params)
    if params then
    	self.avstar:setPosition(params.pos)
    	params.parent:addChild(self.avstPlayerar)
    end
end


function Player:onExit()
    
end

return Player