--*********************************************
-- Desc: 装备系统
-- File: EquipSystem.lua
-- Author: zhouhaifeng
-- Date: 2014-03-31 18:53:23
--*********************************************

local EquipData = import("app.datas.GameItems")
local Equip = import("app.utils.GameData")
local EquipSystem = {}
local KEY = "Equip"

local equipConfigModel = {
	itemId = nil,
	isEquip = true,
	num = 0,
	name = "",
	useTime = 0,
}

local defaultEquips = {
	1000,
	1001,
	2000,
	2001,
	3001,
	4000,
	4001,
	4003,
}


--[[
	初始化装备信息
	只有第一次进入的时候会调用
]]
function EquipSystem:init()
	--全部的装备
	self._equip = {}	
	
	self._equip[kItemTypeCars] = {}
	self._equip[kItemTypeWeapon] = {}
	self._equip[kItemTypeClothes] = {}
	self._equip[kItemTypeProps] = {}
	self._equip[kItemTypeTemp] = {}
	for k,v in pairs(defaultEquips) do
		self:buyItem(v,true)
	end

	self:save()
end

--[[
	获取最喜欢的道具
]]
function EquipSystem:getFavouriteItem(itemType)
	local prop = nil
	local itemId = nil
	local maxUseTime = 0
	--遍历某种类型的道具
	if itemType then
		local datas = self._equip[itemType]
		if datas then
			for k,v in pairs(datas) do
				maxUseTime = math.max(maxUseTime,v.useTime)
				if maxUseTime == v.useTime then
					itemId = v.itemId
					prop = v
				end
			end
		end

		return prop
	else
	--遍历所有类型的道具
		for equipType,equipList in pairs(self._equip) do
			if equipList then
				for k,v in pairs(equipList) do
					maxUseTime = math.max(maxUseTime,v.useTime)
					if maxUseTime == v.useTime then
						itemId = v.itemId
						prop = v
					end
				end
			end
		end

		return prop

	end
end

--[[
	获取装备道具属性
]]
function EquipSystem:getEquipProp(itemId)
	local data = EquipData[itemId]	
	if data then	
		return clone(data)
	end
	return nil
end

--[[
	设置是使用次数
]]
function EquipSystem:setUseTime()
	local list = self:getEquipList()
	if list then
		for k,v in pairs(list) do
			v.useTime = v.useTime + 1
		end
	end
	self:save()
end

--[[
	设置是使用次数
]]
function EquipSystem:getEquipList()
	local list = {}
	for equipType,equipList in pairs(self._equip) do
		if equipList then
			for k,v in pairs(equipList) do
				if v.isEquip then
					table.insert(list, tostring(v.itemId) ,v)
				end
			end
		end
	end
	return list
end

--[[
	获取已经装备
]]
function EquipSystem:getEquipItemWithType(itemType)

	local list = self._equip[itemType]
	
	if list ~= nil then
		for k,v in pairs(list) do
			if v.isEquip then
				return v.itemId
			end
		end
	end

	
end

--[[
	消耗型道具使用
]]
function EquipSystem:useEquipItem(itemId)

end


function EquipSystem:getItemTypeTable(itemType)
	local function sortEquipIdAsc(a,b)
		return a.itemId < b.itemId
	end

	local _table = clone(EquipData)
	table.filter(_table, function(v,k)
		return (v.itemType == itemType)
	end)

	_table = table.values(_table)

	table.sort(_table,sortEquipIdAsc)

	return _table
end

--[[
	装备道具
]]
function EquipSystem:equipItem(itemId)

	if not self:checkIsExist(itemId) then
		-- CCLuaLog("装备套装不存在")
		return false
	end
	
	local data = EquipData[itemId]		
	if data.itemType == kItemTypePackage then
		for k,v in pairs(data.content) do
			self:equipItem(v)
		end
	else
		local prop = self._equip[data.itemType][tostring(itemId)]  
		if prop ~= nil then
			if prop.isEquip == false then
				self:unequipItemWithType(data.itemType)
				prop.isEquip = true
				-- CCLuaLog("============================>【" .. data.name .."】装备成功")
			else
				-- CCLuaLog("装备道具已经是使用状态")
			end
		else
			-- CCLuaLog("没有这个装备道具...")
		end
	end

	return true
end

--[[
	获取道具数量
]]
function EquipSystem:getPropNum(itemId)

	local prop = self:checkIsExist(itemId) 
	if prop then
		return prop.num
	end
	return 0
end

--[[
	设置道具数量
]]
function EquipSystem:setPropNum(itemId,num)
	local prop = self:checkIsExist(itemId) 
	if prop then
		prop.num = num
	end
	self:save()
end

--[[
	卸下当前的所有装备,readOnly
]]
function EquipSystem:unEquipAllItems()
	local list = self:getEquipList()
	if list then
		for k,v in pairs(list) do
			unEquipItem(v.itemId)
		end
	end
end

--[[
	卸下当前的装备
]]
function EquipSystem:unEquipItem(itemId)
	local data = EquipData[itemId]
	local prop = self._equip[data.itemType][tostring(itemId)]  
	
	if prop ~= nil then
		if prop.isEquip then
			prop.isEquip = false
			-- CCLuaLog("<============================【" .. data.name .."】卸载成功")
		else
			-- CCLuaLog("装备道具不是使用状态")
		end
	else
		-- CCLuaLog("没有这个装备道具...")
	end
end

--[[
	卸下当前的装备
]]
function EquipSystem:unequipItemWithType(itemType)
	local list = self:getEquipList()
	if list then
		for k,v in pairs(list) do
			local data = EquipData[v.itemId]
			if data.itemType == itemType then
				self:unEquipItem(v.itemId)
				return 
			end
		end
	end
end

--[[
	判断是否已经拥有该装备道具
]]
function EquipSystem:checkIsExist(itemId)
	local data = EquipData[itemId]

	if data.itemType == kItemTypePackage then
		for k,v in pairs(data.content) do
			if not self:checkIsExist(v) then
				-- echoInfo(data.name .. "--套装不存存在")
				return false
			end
		end
		-- echoInfo(data.name .. "--套装已经存在")
		return true
	-- 单件物品
	else 
		local prop = self._equip[data.itemType][tostring(itemId)]  
		if prop ~= nil then
			-- echoInfo(prop.name .. "--装备道具已经存在")
			return prop
		else
			-- echoInfo(data.name .. "--装备不存在")
			return false
		end
	end
end


--[[
	获得一件装备
]]
function EquipSystem:buyItem(itemId,equip)
	local isEquip = equip or false
	echoInfo("buyItem:" .. itemId)
	local data = EquipData[itemId]

	-- CCLuaLog("buyItem" .. data.name .."(" ..itemId ..")")
	--套装
	if data.itemType == kItemTypePackage then
		for k,v in pairs(data.content) do
			local prop = EquipData[v]
			-- CCLuaLog("----+buyItem" .. prop.itemId)
			local list = self._equip[prop.itemType] 
			local config = clone(equipConfigModel)
			config.itemId = prop.itemId
			config.isEquip = isEquip
			config.name = prop.name
			if list[prop.itemId] then
				-- CCLuaLog("已经拥有该物品了，TODO：退还多少钱...")
			end
			list[tostring(prop.itemId)] = config		
		end
	-- 单件物品
	else 
		local list = self._equip[data.itemType] 
		local config = clone(equipConfigModel)
		config.itemId = itemId
		config.isEquip = isEquip
		config.name = data.name

		if data.itemType == kItemTypeProps then

			config.num = EquipSystem:getPropNum(config.itemId) + data.quantity
		end
		list[tostring(itemId)] = config
	end

	self:save()
end

--[[
	检测装备购买限制
]]
function EquipSystem:checkIsInLimit(itemId)
	local data = EquipData[itemId]
	--有等级军衔限制
	if data.levelLimit ~= nil then
		local level = LevelSystem:getCurrentLevel()
		 if data.levelLimit > level then
		 	return true
		 end
	end

	return false
end

--[[
	加载装备信息
]]
function EquipSystem:load()
	Equip.init(KEY,function(event)
			if event.name == "load" then
				if event.errorCode ~= nil then
					if event.errorCode == ERROR_STATE_FILE_NOT_FOUND then
						self:init()
					end
				else
					if event.values ~= nil and type(event.values) == "table" then
						self._equip = clone(event.values)
						-- CCLuaLog(KEY .. " load success")
					else
						self:init()
					end
				end
			elseif event.name == "save" then
				-- CCLuaLog(KEY .. " save success!")
				return event.values
			end
	end,"equip.dat",KEY)

	Equip.load(KEY)
end

--[[
	保存装备信息
]]
function EquipSystem:save()
	Equip.save(KEY,self._equip)
end

return EquipSystem
