--******************************************** 
-- Desc:加载工具类
-- File：LoadingHelper.lua
-- Author: zhouhaifeng
-- Date: 2014-03-18 22:32:40
--********************************************

local LoadingHelper = {}
local isLoading = false 	--是否正在加载
local preLoadTables = {}	--需要预加载的资源
local currentResCount = 0 	--当前加载到第几个
local totalResCount = 0 	--总共需要加载几个
local handler = nil
--资源类型
local PRE_RES_TYPE = {
	IMAGE_TEXTURE = 1,		--图片纹理
	SPRITE_FRAME = 2,		--动画帧
	OBJECT_POOLS = 3,		--对象池
	MUSIC_FILE = 4,			--背景音乐文件
	EFFECT_FILE = 5,		--音效文件
}

function LoadingHelper:getExtension(filename)
	return filename:match(".+%.(%w+)$")
end

--[[
	回调函数
]]
function loadingCallbackHandler(medthod,texture,fileName,loadState)
	currentResCount = currentResCount + 1
	
	echoInfo("<=======preLoading step:(" .. currentResCount .. "/" ..totalResCount ..") medthod:" ..medthod .. " state:"..loadState .. 
			"=======>")

	if currentResCount == totalResCount then
		echoInfo("<===============================END PRELOADING===============================>")
	end

	if handler then
		handler(currentResCount,totalResCount)	
	end
end

function AsyTextureCallback(medthod,texture)
	if texture then
		loadingCallbackHandler("addImageAsync",nil,nil,"finish")
	else
		loadingCallbackHandler("addImageAsync",nil,nil,"skip")
	end
	
end
--[[
	设置回调函数
]]
function LoadingHelper:setStepCallbackHandler(callbackHandler)
	if callbackHandler ~=nil then
		handler = callbackHandler
	end
end

--[[
	设置需要预加载的资源
	preResTables: 资源列表
]]
function LoadingHelper:preLoadingAll(preResTables)
	if isLoading then
		return false
	end

	if preResTables == nil then
		return false
	end

	local _preResTables = preLoadTables[PRE_RES_TYPE.IMAGE_TEXTURE]
	if _preResTables == nil then
		_preResTables = {}
	end
	table.insertTo(_preResTables, preResTables)
	preLoadTables[PRE_RES_TYPE.IMAGE_TEXTURE] = _preResTables
end

--[[
	设置需要预加载的资源
	resType: 资源类型
	preResTables: 资源列表
]]
function LoadingHelper:preLoading(resType,preResTables)
	if isLoading then
		return false
	end

	if preResTables == nil then
		return false
	end

	if not table.find(PRE_RES_TYPE, resType) then
		return false
	end

	local _preResTables = preLoadTables[resType]
	if _preResTables == nil then
		_preResTables = {}
	end

	table.insertTo(_preResTables, preResTables)
	preLoadTables[resType] = _preResTables
end

--[[
	开始加载
]]
function LoadingHelper:start()
	echoInfo("<===============================START PRELOADING===============================>")

	--初始化
	isLoading = true
	currentResCount = 0
	totalResCount = 0
	local _textureTables = preLoadTables[PRE_RES_TYPE.IMAGE_TEXTURE] or {}
	local _spriteframeTables = preLoadTables[PRE_RES_TYPE.SPRITE_FRAME] or {}
	local _objectpoolsTables = preLoadTables[PRE_RES_TYPE.OBJECT_POOLS] or {}
	local _musicfileTables = preLoadTables[PRE_RES_TYPE.MUSIC_FILE] or {}
	local _effectfileTables = preLoadTables[PRE_RES_TYPE.EFFECT_FILE] or {}

	totalResCount = totalResCount + #_textureTables;
	totalResCount = totalResCount + #_spriteframeTables;
	totalResCount = totalResCount + #_objectpoolsTables;
	totalResCount = totalResCount + #_musicfileTables;
	totalResCount = totalResCount + #_effectfileTables;
	
	--预加载图片纹理
	for key,fileName in pairs(_textureTables) do 
		local ext = self:getExtension(fileName)
		ext = string.lower(tostring(ext))
		if  ext == "png" or ext == "jpg" then
			display.addImageAsync(fileName,AsyTextureCallback,"finish")
		else
			loadingCallbackHandler("addImageAsync",nil,fileName,"skip")
		end
	end

	--预加载序列帧动画	
	for key,objects in pairs(_spriteframeTables) do 
		local plist_ext = self:getExtension(objects.plistFile)
		local image_ext = self:getExtension(objects.imageFile)
		ext = string.lower(tostring(ext))
		if plist_ext == "plist" and image_ext == "png" or image_ext == "pvr" or image_ext == "pvr.ccz" then
			display.addSpriteFramesWithFile(objects.plistFile,objects.imageFile)
			loadingCallbackHandler("addSpriteFrameCache",nil,objects.plistFile,"finish")
		else
			loadingCallbackHandler("addSpriteFrameCache",nil,objects.plistFile,"skip")
		end
	end

	--预加载对象池
	for key,objects in pairs(_objectpoolsTables) do 
		if type(objects) == "table" then
			if  Global:get(objects.className) == nil then
				local pools = require(objects.packageName).new()
				Global:set(objects.className,pools)
				loadingCallbackHandler("addObjectPools",nil,objects.className,"finish")			
			else
				loadingCallbackHandler("addObjectPools",nil,objects.className,"skip")
			end
		else
			loadingCallbackHandler("addObjectPools",nil,objects.className,"skip")
		end
	end

	--预加载背景音乐
	for key,fileName in pairs(_musicfileTables) do 
		local ext = self:getExtension(fileName)
		ext = string.lower(tostring(ext))
		if ext == "mp3" or ext == "ogg" or ext == "acf" then
			audio.preloadMusic(fileName)
			loadingCallbackHandler("preloadMusic",nil,fileName,"finish")
		else
			loadingCallbackHandler("preloadMusic",nil,fileName,"skip")
		end
	end

	--预加载背景音效
	for key,fileName in pairs(_musicfileTables) do 
		local ext = self:getExtension(fileName)
		ext = string.lower(tostring(ext))
		if ext == "mp3" or ext == "ogg" or ext == "acf" then
			audio.preloadSound(fileName)
			loadingCallbackHandler("preloadSound",nil,fileName,"finish")
		else
			loadingCallbackHandler("preloadSound",nil,fileName,"skip")
		end
	end

	isLoading = false
end

--[[
	重置
]]
function LoadingHelper:reset()
	isLoading = false 	--是否正在加载
	preLoadTables = {}	--需要预加载的资源
	currentResCount = 0 	--当前加载到第几个
	totalResCount = 0 	--总共需要加载几个
	handler = nil
end

--[[
	判断是否正在加载
]]
function LoadingHelper:isLoading()
	return isLoading
end

return LoadingHelper
