--*********************************************
-- Desc: CoverFlow控件
-- File: CoverFlow.lua
-- Author: zhouhaifeng
-- Date: 2014-04-10 09:47:46
--*********************************************

local kCoverFlowHorizontal = 0
local kCoverFlowVerVertical = 1

local CoverFlowCell = import("app.utils.ui.CoverFlowCell")

local CoverFlow = class("CoverFlow", function(rect,direct)
    if not rect then rect = CCRect(0, 0, 0, 0) end
    local node = display.newClippingRegionNode(rect)
    node:setNodeEventEnabled(true)
    require("framework.api.EventProtocol").extend(node)
    return node
end)

--[[
	设置
]]
function CoverFlow:ctor(rect,direct)
	self:setUp(rect,direct)
end

function CoverFlow:setDataSource(data)
	self.data = data
	self:layoutCells()
end

--[[
	设置cell
]]
function CoverFlow:layoutCells()

	for i = 1 , #self.data do
		local v = self.data[i]
		self.cardItems[i] = CoverFlowCell.new(60 , i , self.direct, v.itemId)
		self.cardItems[i].index = i
		--self.cardItems[i].textLabel:setString("【" .. i .. "】" ..v.name)
		self.cardItems[i]:addTo(i,self)
	
	end

	local index = 1
	local count = #self.data
	for k, v in pairs(self.cardItems) do
		v:setCellCount(count)
		self:callEventHandler({coverflow = self, event = "setupCells", currentCell = v})
	end
end

--[[
	移动到某格中
]]
function CoverFlow:ScrollToCells(index)
	for k, v in pairs(self.cardItems) do
		v:setState(index,self)
	end
end

function CoverFlow:setUp(rect,direct)
	--触摸范围，显示范围
	self.touchRect = rect
	--CoverFlow方向
	self.direct = direct

	--判断是点击还是滑动 
	self.dragThreshold = 40
	self.isTap = false

	--滑动中需要用到的参数
	self.touchPt = 0
	--是否允许
	self.legal = true
	--上次触摸的位置
	self.lastPt = 0
	self.time = 0
	self.speed = 0
	--是否可以滑动
	self.autoScroll = false
	self.move = false
	self.allow = true

	--记录位置
	self.currentIndex = 1

	--Cell数组
	self.cardItems = {}

	self.view = display.newLayer()
    self:addChild(self.view)

	self.view:setTouchEnabled(true)
    self.view:addTouchEventListener(function(event, x, y)
        return self:onTouchEvent(event, x, y)
    end)

end

--[[
	获取当前
]]
function CoverFlow:getCurrentIndex()
	return self.currentIndex
end

--[[
	返回当前的cell
]]
function CoverFlow:getCurrentCell()
	return self.cardItems[self.currentIndex]
end

--[[
	设置当前第几页
]]
function CoverFlow:setCurrentIndex(index)
	if not self.autoScroll then
		self:callEventHandler({coverflow = self, event = "dataWillChange", currentCell = self:getCurrentCell(),newValue = index , oldValue = self.currentIndex})
	end
	self.currentIndex = index
	if not self.autoScroll then
		self:callEventHandler({coverflow = self, event = "dataDidChange", currentCell = self:getCurrentCell(), newValue = index})
	end
end

--[[
	设置回调
]]
function CoverFlow:setCellEventHandler(callback)
	self.handler = callback
end

--[[
	执行回调
]]
function CoverFlow:callEventHandler( ... )
	if self.handler then
		self.handler( ... )
	end
end

--[[
	触摸事件
]]
function CoverFlow:onTouchEvent(event,x,y)

	local pos = self:convertToNodeSpace(ccp(x,y))
	x,y = pos.x,pos.y 

	--不在范围内直接跳过
	if event == "began" then
		if not self.touchRect:containsPoint(pos) then return false end
		self:onTouchBegan(x, y)
	elseif event == "moved" then	
		self:onTouchMoved(x, y)
	else
		self:onTouchEnded(x, y)
	end
	return true
end

--[[
	点击事件
]]
function CoverFlow:onTaped(index,x,y)
	self:callEventHandler({coverflow = self, event = "onTaped", currentIndex = index , currentCell = self:getCurrentCell(), x = x, y = y})
end

--[[
	触摸开始
]]
function CoverFlow:onTouchBegan(x, y)
	if self.direct == kCoverFlowHorizontal then
		self.touchPt = x
		self.lastPt = x		
	else
		self.touchPt = y
		self.lastPt = y
	end
	self.isTap = true
	self.legal = true
	self.time = os.clock()
end

--[[
	触摸移动
]]
function CoverFlow:onTouchMoved(x, y)

	local nowPt = 0

	if self.direct == kCoverFlowHorizontal then
		nowPt = x
	else
		nowPt = y
	end

	local dir = nowPt < self.lastPt

	if math.abs(self.touchPt - nowPt) > self.dragThreshold then
		self.touchPt = nowPt
		self.speed = os.clock() - self.time + 0.1
		self.time = os.clock()
		self.isTap = false
		if self.allow then
			self.move = true
			self.allow = false
			--下面是不进行循环移动的判断
			if dir then
				if self.cardItems[1].pos > -(#self.cardItems - 4) then
					self.move = true
					self.allow = false
				end
			else
				if self.cardItems[1].pos < 3 then
					self.move = true
					self.allow = false
				end
			end
		end
		self.legal = false		
	else
		self.isTap = true
	end


	
	if self.move then
		self.move = false
		for i = 1, #self.cardItems do
			self.cardItems[i]:move(dir,self,self.speed,true)
		end
	end
	self.lastPt = nowPt
end

--[[
	滚动到第几个
]]
function CoverFlow:moveToIndex(index)
	local total = #self.cardItems
	-- CCLuaLog("=============================>>>>>>>>>>>>>>>>>>>>>>>>>>>>>" .. index  .. "  " .. math.fmod(total - (index - 2),total))
	
	if index <=total and index > 0 then
		self.autoScroll = true
		for i = 1, total do
			self.cardItems[i]:setState(math.fmod(total - (index - 2), total),self)
		end
		self.autoScroll = false
	end
end

--[[
	触摸结束
]]
function CoverFlow:onTouchEnded(x, y)
	local pos = CCPoint(x, y)
	--判断是不是点击
	if self.isTap and self.allow then
		if self.touchRect:containsPoint(pos) then  
			local cell = self:getCurrentCell()
			-- CCLuaLog("pos:(" .. x  .. "," .. y .. ")")
			local rect = cell:getTouchRect()
			-- CCLuaLog("rect:(" .. rect.origin.x  .. "," .. rect.origin.y  .. "," .. rect.size.width .. ","  .. rect.size.height .. ")")
			if cell:getTouchRect():containsPoint(pos) then  
				self:onTaped(self:getCurrentIndex(),x,y)
			end
		end
		self.isTap = false	
	end

	self.touchPt = 0
	self.lastPt = 0
	self.legal = false	
end

return CoverFlow