	--*********************************************
-- Desc: CoverFlow 效果
-- File: CoverFlowCell.lua
-- Author: zhouhaifeng
-- Date: 2014-04-02 17:51:03
--*********************************************


local CoverFlowCell = class("CoverFlowCell")

local kCoverFlowHorizontal = 0
local kCoverFlowVerVertical = 1

local PATH = "ui/level/"
local ICONPATH = "ui/level/"
-- 卡片在不同位置上的锚点，坐标等信息

--横版
local posInfo_h = {
	--不同位置的锚点，x坐标，旋转角度，z轴，缩放比例,alpha
	[0] = {ccp(0,0.5), -20, -40, 0,0.75,0.4},
    {ccp(0,0.5),   0, -40, 1, 0.85,0.6},
	{ccp(0,0.5),  20,  -40, 2, 0.9,0.8},
	{ccp(0,0.5), 140,   0, 3,  1 ,1},
	{ccp(1,0.5), 440,  40, 2, 0.9,0.8},
	{ccp(1,0.5), 460,  40, 1, 0.85,0.6},
	{ccp(1,0.5), 480,  40, 0, 0.75,0.4},
	{ccp(1,0.5), 500,  40, -1, 0.6,0.4},
	{ccp(1,0.5), 500,  40, -2, 0.55,0.4},
	{ccp(1,0.5), 500,  40, -3, 0.50,0.4},
	{ccp(1,0.5), 500,  40, -3, 0.50,0.4},
}

--竖版
local posInfo_v = {
	--不同位置的锚点，x坐标，旋转角度，z轴，缩放比例,alpha
	[0] = {ccp(0.5,1),   70, -45, 1, 0.6,0.6},
	{ccp(0.5,1),  80,  -45, 2, 0.8,0.8},
	{ccp(0.5,0.5), 100,   0, 3,  1 ,1},
	{ccp(0.5,0), 120,  45, 2, 0.8,0.8},
	{ccp(0.5,0), 120,  45, 1, 0.6,0.6},
	{ccp(0.5,0), 120,  45, 0, 0.5,0.4},
	{ccp(0.5,0), 120,  45, -1, 0.4,0.4},
	{ccp(0.5,0), 120,  45, -2, 0.4,0.4},
	{ccp(0.5,0), 120,  45, -3, 0.4,0.4},
	{ccp(0.5,0), 120,  45, -3, 0.4,0.4},
	{ccp(0.5,0), 120,  45, -3, 0.4,0.4},
}


local function tintTo(target , args)
   local action
   if args then
    	 action = CCTintTo:create(args.time , args.r , args.g , args.b)
     else
    	 action = CCTintTo:create(0,255,255,255 )
     end
    return transition.execute(target, action, args)
end

--[[ 获取所有子节点里的 CCSprite ]]
local function getAllSprites( root )
	local sprites = {}

	local function _getAllSprites( _root )
		local childs_num = _root:getChildrenCount()
		if childs_num == 0 then return end

		local childs = _root:getChildren()
		for i = 0 , childs_num - 1 do
			local child = tolua.cast( childs:objectAtIndex(i) , "CCNode")
			sprites[#sprites + 1] = tolua.cast( child , "CCSprite")
			_getAllSprites(child)
		end
	end

	_getAllSprites( root )

	return sprites
end

local function playSprites(target , actionName , args)
	local sprites = getAllSprites( target )
	if #sprites == 0 then return end

	local first_complete = true	-- 保证 onComplete 只执行一次
	for i = 1 , #sprites do
		if first_complete == true and args.onComplete then
			first_complete = false
		else
			args.onComplete = nil
		end
		tintTo( sprites[i] , args)
	end
end

--设置锚点与位置,x,y默认为0，锚点默认为0
local function setAnchPos(node,x,y,anX,anY)
	local posX , posY , aX , aY = x or 0 , y or 0 , anX or 0 , anY or 0
	node:setAnchorPoint(ccp(aX,aY))
	node:setPosition(ccp(posX,posY))
end

function CoverFlowCell:ctor(line,pos,direct,itemId)

	self.layer = nil
	self.id = nil
	self.pos = nil
	self.rotate = nil
	self.light = nil
	self.count = 6
	self.posInfo = nil
	self.itemId = itemId
	self.layer = CCLayer:create()

	
	self.pos = pos 
	self.id = pos

	local str = ""
	self.direct = direct

	if self.direct == kCoverFlowHorizontal then
		str = PATH.."card_bg.png"
		self.posInfo = posInfo_h
	else
		
		str = PATH .."ui_item_".. itemId ..".png"
		self.posInfo = posInfo_v
	end
	-- CCLuaLog(str);
	self.bg = display.newSprite(str)
	self.bg:setAnchorPoint(ccp(0,0))
	self.bg:setPosition(ccp(0,0))
	-- CCLuaLog("pos:" .. self.pos);
	setAnchPos(self.bg)
	
	self.layer:setContentSize(self.bg:getContentSize())
	self.layer:ignoreAnchorPointForPosition(false)	

	if self.direct == kCoverFlowHorizontal then
		self.centerPos = 3
		setAnchPos(self.layer,self.posInfo[self.pos][2],line,0.5,0.5)
	else
		self.centerPos = 2
		setAnchPos(self.layer,line,self.posInfo[self.pos][2],0.5,0.5)
	end
	
	self.layer:addChild(self.bg)
		
	function self.layer:getRotate()
		return self.rotate
	end

	local width = self.layer:getContentSize().width
	local height = self.layer:getContentSize().height 

	self.lock = display.newSprite("ui/level/ui_kaiqi_0.png",width/2,height/2)
	self.lock:setVisible(false)
	self.layer:addChild(self.lock)

	
	-- self.textLabel = ui.newTTFLabel({text = "第"..self.id .. "个" , size = 12 , color = ccc3(0,0,0)})
	-- setAnchPos(self.textLabel,width / 2, 2, 0.5)
	-- self.layer:addChild(self.textLabel)
end

function CoverFlowCell:setCellCount(count)
	self.count = count
	-- body
end

function CoverFlowCell:getTouchRect()
	local x,y = self.bg:getPosition()
	local size = self.bg:getContentSize()
	return CCRectMake(x, y + 50, x + size.width, y + size.height + 50)
end

function CoverFlowCell:getLayer()
	return self.layer
end

function CoverFlowCell:addTo(i,parent)
	self.rotate = self.posInfo[i][3] 
	self.layer:setScale(self.posInfo[i][5])
	self.layer:setOpacity(self.posInfo[i][6])
	self.layer:setAnchorPoint(self.posInfo[i][1])
	
	--使用摄像机进行卡牌的翻转效果实现
	if self.direct == kCoverFlowHorizontal then
		self.layer:runAction(CCOrbitCamera:create(0,1,0,self.rotate,0,0,0))
	else
		self.layer:runAction(CCOrbitCamera:create(0,1,0,0,0,60,0))
	end
	
	if i ~= self.centerPos then
		playSprites(self.layer,"tintTo",{time = 0, r = 100,g=100,b=100})
	else
		self:showLight(true)
	end
	parent:addChild(self.layer,self.posInfo[i][4])
end


--直接设置卡牌的位置状态
function CoverFlowCell:setState(step,parent)
	self.pos = self.pos + step
	if self.pos > self.count then
		self.pos = self.pos - self.count
	end 

	self.rotate = self.posInfo[self.pos][3] 
	self.layer:setScale(self.posInfo[self.pos][5])
	self.layer:setAnchorPoint(self.posInfo[self.pos][1])
	

	local alpha = self.posInfo[self.pos][6]
	--使用摄像机进行卡牌的翻转效果实现
	if self.direct == kCoverFlowHorizontal then
		self.layer:setPositionX(self.posInfo[self.pos][2])
		self.layer:runAction(CCOrbitCamera:create(0,1,0,self.rotate,0,0,0))
	else
		self.layer:setPositionY(self.posInfo[self.pos][2])
		self.layer:runAction(CCOrbitCamera:create(0,1,0,0,0,60,0))
	end
	

	playSprites(self.layer,"tintTo",{time = 0, r = 255 * alpha,g=255 * alpha,b=255 * alpha})

	parent:reorderChild(self.layer,self.posInfo[self.pos][4])
	if self.pos == self.centerPos then
		-- DATA_Formation:setCur(self.id)
		parent:setCurrentIndex(self.id)
		self:showLight(true)
	else
		self:showLight(false)
	end
end


function CoverFlowCell:move(left,parent,time,anim)
	local time, startA, desA, posX, anchCha, anchX,scale,zOrder = time, 0, 0, 0, 0, ccp(0,0.5),0.8,0

	if anim then
		if time < 0.1 then
			time = 0.1
		elseif time > 0.2 then
			time = 0.2
		end
	else
		time = 0
	end
	
	
	local array = CCArray:create()
	local sequence = CCArray:create()
	
	local next
	if left then
		---------------卡片循环移动的设置---------
		if self.pos == 0 then
			self.pos = self.count
			self.rotate = self.posInfo[self.pos][3] 
			self.layer:setScale(self.posInfo[self.pos][5])
			self.layer:setAnchorPoint(self.posInfo[self.pos][1])
			if self.direct == kCoverFlowHorizontal then
				self.layer:setPositionX(self.posInfo[self.pos][2])
			else
				self.layer:setPositionY(self.posInfo[self.pos][2])
			end
		end
		-----------------------
		next = self.pos - 1	
		if next < 0 then
			next = 0
		end
	else
		--下面是要做循环移动的设置
		if self.pos == self.count then
			self.pos = 0
			self.rotate = self.posInfo[self.pos][3] 
			self.layer:setScale(self.posInfo[self.pos][5])
			self.layer:setAnchorPoint(self.posInfo[self.pos][1])
			if self.direct == kCoverFlowHorizontal then
				self.layer:setPositionX(self.posInfo[self.pos][2])
			else
				self.layer:setPositionY(self.posInfo[self.pos][2])
			end
		end
		----------------以上是做循环设置---------------------------------------
		next = self.pos + 1
		if next > self.count then
			next = self.count
		elseif next < 0 then
			next = 0
		end
	end
	
	local anchCha
	local posX = self.posInfo[next][2]
	local anchX = self.posInfo[next][1]
	if self.direct == kCoverFlowHorizontal then
		anchCha = self.layer:getAnchorPoint().x - anchX.x
	else
		anchCha = self.layer:getAnchorPoint().y - anchX.y
	end
	local desA = self.posInfo[next][3]- self.rotate
	local zOrder = self.posInfo[next][4]
	local scale = self.posInfo[next][5]
	local alpha = self.posInfo[next][6]

	if next == self.centerPos then
		self:showLight(true)
	else
		self:showLight(false)
		if not left and next == self.centerPos + 1 then  --当向右滑动时，第三张卡片滑到第四张卡片需要进行亮度，坐标及锚点位置的处理，其余卡牌不用
			
			if self.direct == kCoverFlowHorizontal then
				anchX.x = 1
				anchCha = 0
				self.layer:setAnchorPoint(anchX)
				self.layer:setPositionX(self.layer:getPositionX() + self.layer:getContentSize().width)
			else
		
			end
			
		end
	end

	playSprites(self.layer,"tintTo",{time = time, r = 255 * alpha, g=255 * alpha , b=255 *alpha})
	------------------以上的处理方式是根据滑动方向与卡牌位置进行处理------------------------------------------------
	
	parent:reorderChild(self.layer,zOrder)
	if self.direct == kCoverFlowHorizontal then
		array:addObject(CCOrbitCamera:create(time,1,0,self.rotate,desA,0,0))
		array:addObject(CCMoveTo:create(time,ccp(posX + self.layer:getContentSize().width * anchCha,self.layer:getPositionY())))
	else
		array:addObject(CCOrbitCamera:create(time,1,0,0,0,self.rotate,desA))
		array:addObject(CCMoveTo:create(time,ccp(self.layer:getPositionX(), posX + self.layer:getContentSize().height * anchCha)))
	end
	
	
	array:addObject(CCFadeTo:create(time,alpha))
	array:addObject(CCScaleTo:create(time,scale))
	sequence:addObject(CCSpawn:create(array))	
	sequence:addObject(CCCallFunc:create(function()
		self.layer:setAnchorPoint(anchX)
		if self.direct == kCoverFlowHorizontal then
			self.layer:setPositionX(self.layer:getPositionX() - self.layer:getContentSize().width * anchCha)
		else
			self.layer:setPositionY(self.layer:getPositionY() - self.layer:getContentSize().height * anchCha)
		end

		self.rotate = desA + self.rotate
		if left then
			self.pos = self.pos - 1
		else
			self.pos = self.pos + 1
		end
		parent.allow = true 
		 --动作完成后，允许下次的移动
		if self.pos == self.centerPos then
			parent:setCurrentIndex(self.id)
		end
		
	end))
	self.layer:runAction(CCSequence:create(sequence))
end

function CoverFlowCell:showLight(bool)
	-- if bool then
	-- 	if self.light then
	-- 		self.light:setVisible(true)
	-- 	else			
	-- 		if self.direct == kCoverFlowHorizontal then
	-- 			self.light = display.newSprite(PATH.."card_light.png")
	-- 		else
	-- 			self.light = display.newSprite(PATH.."card_v_light.png")
	-- 		end
	-- 		setAnchPos(self.light,self.layer:getContentSize().width / 2,self.layer:getContentSize().height / 2,0.5,0.5)
	-- 		self.layer:addChild(self.light,-1)
			
	-- 		local function createAction()
	-- 			local action
	-- 			local array = CCArray:create()
	-- 			array:addObject(CCScaleTo:create(1,0.9))
	-- 			array:addObject(CCScaleTo:create(1,0.98))
	-- 			array:addObject(CCCallFunc:create(
	-- 			function()
	-- 				self.light:runAction(createAction())
	-- 			end))
	-- 			action = CCSequence:create(array)
	-- 			return action
	-- 		end
	-- 		self.light:runAction(createAction())
	-- 	end
	-- else
	-- 	if self.light then
	-- 		self.light:setVisible(false)
	-- 	end
	-- end
end

return CoverFlowCell