-- 
-- Desc: 评分控件
-- File: StarRate.lua
-- Author: zhouhaifeng
-- Date: 2014-03-22 00:10:33
--

local StarRate = class("StarRate",function()
	return display.newNode()
end)


function StarRate:ctor( ... )
end

--[[
	设置属性	
	params = {
		max = 3,
		cur = 1,
		on = "star_on.png",
		off = "star_on.png",
		offset = 10
	}

]]
function StarRate:setup(params)
	assert(type(params) == "table",
           "[utils.ui] StarRate.new() invalid params")
    self.max      = params.max or 3
    self.cur     = params.cur or 0
    self.on     = params.on
    assert(self.on and type(self.on) == "string", "[ui.StarRate] invalid params.on")
    self.off    = params.off
    assert(self.off and type(self.off) == "string", "[ui.StarRate] invalid params.off")
    self.half   = params.half 

    self.offset = params.offset or 10  
    local x, y      = params.x, params.y

    self.rates = {}
    local starX = 0
    local starY = 0
    for i = 1 , params.max do
        local star
        if i <= params.cur then
    	   star = display.newSprite(params.on, starX, starY)
        else
           star = display.newSprite(params.off, starX, starY)
        end	
    	 self.rates[i] = star
         self:addChild(star)
         
    	 starX = starX + star:getContentSize().width + self.offset
    end

    if type(x) == "number" and type(y) == "number" then
        self:setPosition(x, y)
    end

    self:setContentSize(CCSizeMake(starX, starY))

	return self
end

--[[
	设置得分
	rate 得分数
	bAnimate 是否有动画
]]
function StarRate:setRate(rate,bAnimate)

    if rate > self.max then
        return 
    end

    self.cur = rate
    local starX = -48
    local starY = 0
    for i = 1 , self.max do
    	 local star = self.rates[i]
         local contentRect = star:getDisplayFrame():getRect()       
         --改变星星
         if i <= self.cur then
             star:setDisplayFrame(CCSpriteFrame:create(self.on,contentRect))
         else
             star:setDisplayFrame(CCSpriteFrame:create(self.off,contentRect))
         end
         --设置位置   
         starY = 0
    	 starX = starX + contentRect.size.width + self.offset
         --是否偶动画
         if bAnimate then
            star:setVisible(false)
            local point = self:convertToNodeSpaceAR(ccp(display.cx, display.cy))
            star:setPosition(point)

            point = ccp(starX,starY)

            local delay = CCDelayTime:create(0.5+i*0.2)
            local callback = CCCallFunc:create(function()
                 if star then
                    star:setVisible(true)
                    star:setScale(1.4)
                    local move = CCMoveTo:create(0.2, point)
                    local scale = CCScaleTo:create(0.2, 0.8)
                    local action = CCSequence:createWithTwoActions(move, scale)
                    star:runAction(action)
                    self.handler = nil
                end
            end)
            local action = CCSequence:createWithTwoActions(delay,callback)
            star:runAction(action)
        end
    end
end


function StarRate:onEner()
	
end

function StarRate:onExit()

end


return StarRate
